unit OrderF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Mask, ToolEdit, CurrEdit, StdCtrls, SoCtrls, ComCtrls, SohoBtns, Grids,
  SoDBGrid, SoGrids, SohoRgst, SoBckgrd, Placemnt, SoRxTls, SoClass, SoFolder,
  Db, DBTables, SoExlRep, SoCtmFld, SoCtmRgs, DBGrids, RXDBCtrl;

type
  TOrderForm = class(TForm)
    sohoLabel1: TsohoLabel;
    sohoLabel2: TsohoLabel;
    sohoLabel3: TsohoLabel;
    NumberE: TsohoEdit;
    OrdDateE: TsohoEdit;
    SummE: TCurrencyEdit;
    PageControl1: TPageControl;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    IncludePage: TTabSheet;
    PositionsPage: TTabSheet;
    sohoLabel4: TsohoLabel;
    sohoLabel5: TsohoLabel;
    KnoGrid: TsohoDBGrid;
    KnoPosGrid: TsohoDBGrid;
    sohoLabel6: TsohoLabel;
    FactoryGrid: TsohoDBGrid;
    PosGrid: TsohoStringGrid;
    Ini: TsohoFormStorage;
    Back: TsohoBackground;
    SubReg: TsohoSubRegister;
    KnoQ: TQuery;
    KnoSource: TDataSource;
    KnoFolder: TsohoFolder;
    KnoPosFolder: TsohoFolder;
    KnoPosSource: TDataSource;
    KnoPosQ: TQuery;
    FactoryFolder: TsohoFolder;
    FactorySource: TDataSource;
    FactoryQ: TQuery;
    PrintB: TsohoBitBtn;
    Reporter: TsohoExcelReporter;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure KnoGridKeyPress(Sender: TObject; var Key: Char);
    procedure KnoPosGridKeyPress(Sender: TObject; var Key: Char);
    procedure FactoryGridKeyPress(Sender: TObject; var Key: Char);
    procedure PosGridCalcCellColors(Sender: TObject; Col, Row: Integer;
      aFont: TFont; var aColor: TColor; State: TGridDrawState);
    procedure PosGridAfterCellEdit(Sender: TObject; Col, Row: Integer);
    procedure PosGridBeforeCellEdit(Sender: TObject; Col, Row: Integer);
    procedure PageControl1Changing(Sender: TObject;
      var AllowChange: Boolean);
    procedure PrintBClick(Sender: TObject);
    procedure KnoPosQFilterRecord(DataSet: TDataSet; var Accept: Boolean);
  private
    { Private declarations }

    procedure SaveOrder;
    procedure PrepareOrder;

    procedure Recalc; //   

    function  NewOrder : boolean;
    function  EditOrder (IdOrder : LongInt) : boolean;

    procedure CreateCrosses; //  cross-cells  
                             //   ""

    procedure AddPosition (Id, IdUnits : LongInt);
    procedure RemovePosition (Id, IdUnits : LongInt);

    procedure AddFactory (Id : LongInt; FactoryName : string);
    procedure RemoveFactory (Id : LongInt);

    procedure CheckEnable;

    function PrintOrder (Part, Parts : integer; var StopPrint : boolean) : boolean;

  public
    { Public declarations }
    FIdOrder : LongInt;
    EditMode : boolean;
    UnitsIds, UnitsIncIds : TsohoIntegerList;
    UnitsNames : TStringList;
    DelPosIds, AddPosIds : TsohoIntegerList;
    DelFacIds, AddFacIds : TsohoIntegerList;
    DelKnoIds, AddKnoIds : TsohoIntegerList;

    PosRemIds : TsohoFloatList;
    PosCost : TsohoFloatList;
    UnitsQuantity : TsohoFloatList;

    LastValue : Double;

    RecalcEnabled : boolean;
  end;

var OrderForm: TOrderForm;

function NewOrder : boolean;
function EditOrder (IdOrder : LongInt) : boolean;

implementation
uses SoUtils, SoDBRtn, Global, SoUnit, Main, SoDate, Clipbrd, SoExlCst, SohoOLE,
     SoExRepD, SoBDERtn;

{$R *.DFM}

function NewOrder : boolean;
begin
  try
    if OrderForm = nil then OrderForm := TOrderForm.Create(Application);
    Result := OrderForm.NewOrder;
  except
    OrderForm.Free;
    OrderForm := nil;
  end;
end;

function EditOrder (IdOrder : LongInt) : boolean;
begin
  try
    if OrderForm = nil then OrderForm := TOrderForm.Create(Application);
    Result := OrderForm.EditOrder(IdOrder);
  except
    OrderForm.Free;
    OrderForm := nil;
  end;
end;

procedure TOrderForm.CheckEnable;
begin
  OkB.Enabled := (KnoPosGrid.Results.Count>0) and (FactoryGrid.Results.Count>0);
  PositionsPage.Visible := OkB.Enabled;
end;

procedure TOrderForm.SaveOrder;
var Index, ColIndex, RowIndex, IdOrderInclude,
    IdFactory, IdKnoPos : LongInt;
    OrdIncQuantity : Double;
    Error : boolean;
begin
  //      
  try
    MainForm.Database.StartTransaction;
    MainForm.StatusBar.Gauge.Progress := 0;
    MainForm.StatusBar.Gauge.MaxValue := KnoPosGrid.Results.Count *
      FactoryGrid.Results.Count + KnoGrid.Results.Count + 30;
    MainForm.StatusBar.Gauge.Caption := '';
  //     
    if not ExecuteSQL(SubReg.PathToSQL+'____.sql', BaseDir, [
      IntToStr(FIdOrder)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
    MainForm.StatusBar.Gauge.AddProgress(10);
  //   
    if not ExecuteSQL(SubReg.PathToSQL+'__.sql', BaseDir, [
      IntToStr(FIdOrder)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
    MainForm.StatusBar.Gauge.AddProgress(10);
   //  
   if EditMode then begin
    if not ExecuteSQL(SubReg.PathToSQL+'_.sql', BaseDir, [
      NumberE.AsString,
      DateToSQLDate(OrdDateE.AsDate),
      FLoatToSQLFLoat(SummE.Value),
      IntToStr(FIdOrder)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
   end
   else
    if not ExecuteSQL(SubReg.PathToSQL+'_.sql', BaseDir, [
      IntToStr(FIdOrder),
      NumberE.AsString,
      DateToSQLDate(OrdDateE.AsDate),
      FLoatToSQLFLoat(SummE.Value)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
   MainForm.StatusBar.Gauge.AddProgress(10);
  //   
    for Index := 0 to pred(KnoGrid.Results.Count) do begin
      if not ExecuteSQL(SubReg.PathToSQL+'___.sql', BaseDir, [
        IntToStr(GetNewId('IBKno.New.OrderKno.')),
        IntToStr(KnoGrid.Results[Index]),
        IntToStr(FIdOrder)
      ]) then begin
        MainForm.Database.Rollback;
        exit;
      end;
      MainForm.StatusBar.Gauge.AddProgress(1);
    end;
  //   
    for ColIndex := 0 to pred(KnoPosGrid.Results.Count) do
      for RowIndex := 0 to pred(FactoryGrid.Results.Count) do begin
        IdOrderInclude := GetNewId('IBKno.New.OrderInclude.');
        OrdIncQuantity := WStrToFloat(PosGrid.Cells[ColIndex*2+2, RowIndex+2], Error);
        IdFactory      := FactoryGrid.Results[RowIndex];
        IdKnoPos       := KnoPosGrid.Results[ColIndex];
        if not ExecuteSQL(SubReg.PathToSQL+'___.sql', BaseDir, [
          IntToStr(IdOrderInclude),
          FloatToSQLFloat(OrdIncQuantity),
          IntToStr(FIdOrder),
          IntToStr(IdFactory),
          IntToStr(IdKnoPos)
        ]) then begin
          MainForm.Database.Rollback;
          exit;
        end;
        MainForm.StatusBar.Gauge.AddProgress(1);
      end;
    MainForm.StatusBar.Gauge.Progress := MainForm.StatusBar.Gauge.MaxValue;
    MainForm.StatusBar.Gauge.Caption := '';
    MainForm.Database.Commit;
  except
    MainForm.Database.Rollback;
  end;
end;

procedure TOrderForm.PrepareOrder;
var TmpQ : TQuery;
    Index,
    ColIndex,
    RowIndex : LongInt;
    CellValue : Double;
begin
  KnoPosGrid.Results.Clear;
  KnoGrid.Results.Clear;
  FactoryGrid.Results.Clear;
  DelPosIds.Clear;
  AddPosIds.Clear;
  DelFacIds.Clear;
  AddFacIds.Clear;
  DelKnoIds.Clear;
  AddKnoIds.Clear;
  UnitsIds.Clear;
  UnitsQuantity.Clear;
  UnitsNames.Clear;
  UnitsIncIds.Clear;
  PosRemIds.Clear;
  PosCost.Clear;


  RefreshDataSet(KnoQ);
  RefreshDataSet(KnoPosQ);
  RefreshDataSet(FactoryQ);


  NumberE.AsString := '';
  OrdDateE.AsDate := Date;
  SummE.Value := 0;

  with PosGrid do begin
    ColCount := 3;
    RowCount := 4;
    for Index := 1 to 3 do Cells[0, Index] := '';
    Cells[1, 2] := '  ';
    Cells[1, 3] := '';
    Cells[2, 0] := Cells[1, 3];
  end;

  if EditMode then begin
    RecalcEnabled := false;
    //   ,   
    TmpQ := GetQuery(SubReg.PathToSQL+'___.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        KnoGrid.Results.Add(FieldByName('IdKno').AsInteger);
        Next;
      end;
      Free;
    end;

    TmpQ := GetQuery(SubReg.PathToSQL+'___.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        LocaleById(KnoPosQ, 'Id', FieldByName('IdKnoPos').AsInteger);
        AddPosition(FieldByName('IdKnoPos').AsInteger,
           KnoPosQ.FieldByName('IdUnits').AsInteger);
        Next;
      end;
      Free;
    end;

    TmpQ := GetQuery(SubReg.PathToSQL+'___.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        AddFactory(FieldByName('IdFactory').AsInteger,
                   GetFactoryName(FieldByName('IdFactory').AsInteger));
        Next;
      end;
      Free;
    end;

    RefreshDataSetFilter(KnoPosQ);
    // 
    TmpQ := GetQuery(SubReg.PathToSQL+'___Id.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      NumberE.AsString := FieldByName('OrdNumber').AsString;
      OrdDateE.AsDate := FieldByName('OrdDate').AsDateTime;
      // SummE.Value := FieldByName('OrdSumm').AsFloat;
      Free;
    end;

    // ,  ,     
    //  ,   ...

    //    !
    //  OrderInclude   

    TmpQ := GetQuery(SubReg.PathToSQL+'____Id.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        ColIndex  := 2 + 2*KnoPosGrid.Results.IndexOf(FieldByName('IdKnoPos').AsInteger);
        RowIndex  := 2 + FactoryGrid.Results.IndexOf(FieldByName('IdFactory').AsInteger);
        CellValue := FieldByName('OrdIncQuantity').AsFloat;
        PosGrid.Cells[ColIndex, RowIndex] := FloatToStrF(CellValue, 15, 3);
        Next;
      end;
      Free;
    end;
  end;
  RecalcEnabled := true;
  Recalc;
  CheckEnable;
end;

function TOrderForm.NewOrder : boolean;
begin
  FIdOrder := GetNewId('IBKno.New.Order.');
  EditMode := false;
  PrepareOrder;
  Result := ShowModal = mrOk;
  if not Result then exit;
  SaveOrder;
end;

function TOrderForm.EditOrder (IdOrder : LongInt) : boolean;
begin
  FIdOrder := IdOrder;
  EditMode := true;
  PrepareOrder;
  Result := ShowModal = mrOk;
  if not Result then exit;
  SaveOrder;
end;

procedure TOrderForm.FormCreate(Sender: TObject);
begin
  if Ini.ReadBool('Options', 'ShowBack', true) then begin
    Back.FillMode := TBitmapFillMode(Ini.ReadInteger('Options','OrderFillMode',ord(fmTiles)));
    Back.FileName := SubReg.PathToImages + Ini.ReadString('Options','OrderBack','OrderBack.gif');
  end;
  DelPosIds   := TsohoIntegerList.Create;
  AddPosIds   := TsohoIntegerList.Create;
  DelFacIds   := TsohoIntegerList.Create;
  AddFacIds   := TsohoIntegerList.Create;
  DelKnoIds   := TsohoIntegerList.Create;
  AddKnoIds   := TsohoIntegerList.Create;
  UnitsIds    := TsohoIntegerList.Create;
  UnitsQuantity := TsohoFloatList.Create;
  UnitsIncIds := TsohoIntegerList.Create;
  PosRemIds   := TsohoFloatList.Create;
  PosCost     := TsohoFloatList.Create;
  UnitsNames  := TStringList.Create;

  KnoQ.SQL.LoadFromFile(SubReg.PathToSQL+'_.sql');
  KnoPosQ.SQL.LoadFromFile(SubReg.PathToSQL+'____.sql');
  FactoryQ.SQL.LoadFromFile(SubReg.PathToSQL+'_.sql');
end;

procedure TOrderForm.FormDestroy(Sender: TObject);
begin
  DelPosIds.Free;
  AddPosIds.Free;
  DelFacIds.Free;
  AddFacIds.Free;
  DelKnoIds.Free;
  AddKnoIds.Free;
  UnitsIds.Free;
  UnitsQuantity.Free;
  UnitsNames.Free;
  UnitsIncIds.Free;
  PosRemIds.Free;
  PosCost.Free;
end;

procedure TOrderForm.FormActivate(Sender: TObject);
begin
  PageControl1.ActivePage := IncludePage;
  IncludePage.Repaint;
end;

procedure TOrderForm.KnoGridKeyPress(Sender: TObject; var Key: Char);
var Id : LongInt;
begin
  if Key<>#32 then exit;
  Key := #00;
  Id := KnoQ.FieldByName('Id').AsInteger;
  with KnoGrid do begin
    if Results.IndexOf(Id)<>-1 then begin
      if not YesNoMsg('        '#13+
      ',     : '+KnoQ.FieldByName('KnoNumber').AsString+#13+
      ' ,  -      ?') then exit
      else Results.Remove(Id);
    end
    else Results.Add(Id);
  end;
  RefreshDataSetFilter(KnoPosQ);
  CheckEnable;
end;

procedure TOrderForm.CreateCrosses;
var Index, SummCount : integer;
begin
  with PosGrid do begin
    for Index := pred(CrossCount) downto 0 do DeleteCrossCells(Index);
    for Index := 0 to pred(KnoPosGrid.Results.Count) do
       AddCrossCells(GetGridRect( Index*2 + 2, 0, Index*2 + 3, 0));
    SummCount := UnitsIds.Count;
    if SummCount > 0 then begin
      AddCrossCells( GetGridRect(ColCount - SummCount - 1, 0, ColCount-1, 0));
      for Index := ColCount - SummCount - 1 to ColCount - 1 do
        Cells[Index, 0] := '';
      Cells[ColCount - SummCount - 1, 0] := '';
    end;
    Cells[ColCount - 1, 1] := '';
  end;
end;

procedure TOrderForm.AddPosition (Id, IdUnits : LongInt);
var ColIndex : LongInt;
    Tmp : TQuery;
    UnitsIndex : LongInt;
begin
  //   . .   "".
  //   .,    ,    
  //  
  // !----!--------!---!---!---!---!
  // ! NN !Factory !       !       !
  // !
  UnitsIncIds.Add(IdUnits);
  UnitsIndex := UnitsIds.IndexOf(IdUnits);
  if UnitsIndex = -1 then begin
    //   ,    
    UnitsIds.Add(IdUnits);
    UnitsNames.Add(KnoPosQ.FieldByName('UnitsName').AsString);
    PosGrid.InsertColumn(PosGrid.ColCount-1);
    PosGrid.Cells[PosGrid.ColCount-2, 1] := UnitsNames[UnitsNames.Count-1];
    UnitsQuantity.Add(0);
  end;

  KnoPosGrid.Results.Add(Id);
  with PosGrid do begin
    ColIndex := ColCount - UnitsIds.Count - 1;
    InsertColumn(ColIndex);
    InsertColumn(ColIndex);
    Cells[ColIndex, 0] := KnoPosQ.FieldByName('Name').AsString;

    if UnitsIndex = -1 then Cells[ColIndex, 1] := UnitsNames[UnitsNames.Count-1]
    else Cells[ColIndex, 1] := UnitsNames[UnitsIndex];

    Cells[ColIndex+1, 1] := '';
  end;
  Tmp := GetQuery(SubReg.PathToSQL+'___.sql',
                 'IBKno_DataBase', [
                 IntToStr(FIdOrder),
                 IntToStr(Id),
                 IntToStr(Id)
                 ]);
  if Tmp = nil then exit;
  with Tmp do begin
    PosRemIds.Add(FieldByName('KnoIncQuantity').AsFloat-
                  FieldByName('OutQuantity').AsFloat);
    PosCost.Add(FieldByName('KnoIncCost').AsFloat);
    PosGrid.Cells[ColIndex, PosGrid.RowCount - 2] := FloatToStrF(
      PosRemIds[PosRemIds.Count-1], 15, 3);
    PosGrid.Cells[ColIndex+1, PosGrid.RowCount - 2] := FloatToStrF(
       FieldByName('KnoIncCost').AsFloat, 15, 3);
    Free;
  end;
  if DelPosIds.IndexOf(Id) <> -1 then DelPosIds.Remove(Id);
  CreateCrosses;
  Recalc;
end;

procedure TOrderForm.RemovePosition (Id, IdUnits : LongInt);
var PosIndex, ColIndex : LongInt;
begin
  PosIndex := KnoPosGrid.Results.IndexOf(Id);
  PosRemIds.Delete(PosIndex);
  PosCost.Delete(PosIndex);
  KnoPosGrid.Results.Delete(PosIndex);
  //  ..   "".  
  //    ,   
  //   
  UnitsIncIds.Remove(IdUnits);
  if UnitsIncIds.IndexOf(IdUnits) = -1 then begin
    // . .  ,    
    //   
    ColIndex := UnitsIds.IndexOf(IdUnits);
    UnitsQuantity.Delete(ColIndex);
    UnitsIds.Remove(IdUnits);
    UnitsNames.Delete(ColIndex);
    ColIndex := PosGrid.ColCount - 1 - ColIndex;
    PosGrid.DeleteColumn(ColIndex);
  end;
  //     ,     
  // !  0 !    1   ! 2 ! 3 ! 4 ! 5 !
  // !----!--------!---!---!---!---!
  // ! NN !Factory ! Goods1! Goods2!
  // !
  ColIndex := 2 + PosIndex*2;
  PosGrid.DeleteColumn(ColIndex);
  PosGrid.DeleteColumn(ColIndex);
  DelPosIds.Add(Id);
  CreateCrosses;
  Recalc;
end;

procedure TOrderForm.KnoPosGridKeyPress(Sender: TObject; var Key: Char);
var Id, IdUnits : LongInt;
    Index : integer;
begin
  if Key<>#32 then exit;
  Key := #00;
  Id := KnoPosQ.FieldByName('Id').AsInteger;
  IdUnits := KnoPosQ.FieldByName('IdUnits').AsInteger;
  Index := KnoPosGrid.Results.IndexOf(Id);
  if Index<>-1 then begin
    if not YesNoMsg('         '#13+
      ' ,  -      ?') then exit
    else RemovePosition(Id, IdUnits);
  end
  else AddPosition(Id, IdUnits);
  CheckEnable;
end;

procedure TOrderForm.Recalc; //   
var ColIndex, RowIndex : LongInt;
    S1 : Double;
    PosSumm, CellValue : Double;
    Error : boolean;
    UnitsIndex : Longint;
begin
  if not RecalcEnabled then exit;

  //   
  with PosGrid do begin
    for ColIndex := 0 to pred(KnoPosGrid.Results.Count) do begin
      S1 := 0; //-
      for RowIndex := 2 to (RowCount-3) do begin
        CellValue := WStrToFloat(Cells[2*ColIndex+2, RowIndex], Error);
        S1 := S1 + CellValue;
        Cells[2*ColIndex+3, RowIndex] := FloatToStrF(CellValue * PosCost[ColIndex],
          15, 3);
      end;

      Cells[2*ColIndex+2, RowCount - 1] := FloatToStrF(S1, 15, 3);
      Cells[2*ColIndex+3, RowCount - 1] := FloatToStrF(S1 * PosCost[ColIndex], 15, 3);

      Cells[2*ColIndex+2, RowCount - 2] := FloatToStrF(PosRemIds[ColIndex] - S1, 15, 3);
      Cells[2*ColIndex+3, RowCount - 2] := FloatToStrF((PosRemIds[ColIndex] - S1)
        * PosCost[ColIndex], 15, 3);

    end;
    for RowIndex := 2 to RowCount - 1 do begin
      S1 := 0;
      for ColIndex := 0 to pred(UnitsQuantity.Count) do
        UnitsQuantity[ColIndex] := 0;

      for ColIndex := 0 to pred(KnoPosGrid.Results.Count) do begin
        CellValue  :=  WStrToFloat(Cells[2*ColIndex+2, RowIndex], Error);
        UnitsIndex := UnitsIds.IndexOf(UnitsIncIds[ColIndex]);
        PosSumm    := CellValue*PosCost[ColIndex];
        S1         := S1 + PosSumm;
        UnitsQuantity[UnitsIndex] := UnitsQuantity[UnitsIndex] + CellValue;
      end;
      Cells[ColCount-1, RowIndex] := FloatToStrF(S1, 15, 3);

      for ColIndex := 0 to pred(UnitsQuantity.Count) do
        Cells[ColCount - 1 - UnitsQuantity.Count + ColIndex, RowIndex] :=
          FloatToStrF(UnitsQuantity[ColIndex], 15, 3);
    end;
    SummE.Value := WStrToFloat(Cells[ColCount-1, RowCount -1], Error);
  end;
end;

procedure TOrderForm.RemoveFactory (Id : LongInt);
var RowIndex, Index : integer;
begin
  Index := FactoryGrid.Results.IndexOf(Id);
  FactoryGrid.Results.Delete(Index);
  with PosGrid do begin
    RowIndex := Index + 2;
    DeleteRow(RowIndex);
  end;
  Recalc;
end;

procedure TOrderForm.AddFactory (Id : LongInt; FactoryName : string);
begin
  FactoryGrid.Results.Add(Id);
  with PosGrid do begin
    InsertRow(RowCount - 2);
    Cells[0, RowCount - 3] := IntToStr(FactoryGrid.Results.Count);
    Cells[1, RowCount - 3] := FactoryName;
  end;
  Recalc;
end;

procedure TOrderForm.FactoryGridKeyPress(Sender: TObject; var Key: Char);
var Id, Index : LongInt;
begin
  if Key<>#32 then exit;
  Key := #00;
  Id := FactoryQ.FieldByName('Id').AsInteger;
  Index := FactoryGrid.Results.IndexOf(Id);
  if Index<>-1 then begin
    if not YesNoMsg('         '#13+
      ' ,  -   ?') then exit
    else RemoveFactory(Id);
  end
  else AddFactory(Id, FactoryQ.FieldByName('Name').AsString);
  CheckEnable;
end;

procedure TOrderForm.PosGridCalcCellColors(Sender: TObject; Col,
  Row: Integer; aFont: TFont; var aColor: TColor; State: TGridDrawState);
var Error : boolean;
    Value : Double;
begin
  if (Row>=2) and (Col>=2) then begin
    Value := WStrToFloat(PosGrid.Cells[Col, Row], Error);
    if Value < 0 then begin
      aColor := clRed;
      aFont.Color := clYellow;
      exit;
    end;
    if (Row = PosGrid.RowCount - 2) then aFont.Color := clBlue;
    if (Row = PosGrid.RowCount - 1) then aFont.Style := [fsBold];
  end;
end;

procedure TOrderForm.PosGridAfterCellEdit(Sender: TObject; Col,
  Row: Integer);
var Value : Double;
    Error : boolean;
    DoRecalc : boolean;
begin
  if (Row >= PosGrid.RowCount - 2) or
     (Col >= PosGrid.ColCount - UnitsIds.Count - 1) then
     Value := LastValue;
  Value := WStrToFloat(PosGrid.Cells[Col, Row], Error);
  DoRecalc := Value <> LastValue;
  PosGrid.Cells[Col, Row] := FLoatToStrF(Value, 15, 3);
  if DoRecalc then Recalc;
end;

procedure TOrderForm.PosGridBeforeCellEdit(Sender: TObject; Col,
  Row: Integer);
var Error : boolean;
begin
  LastValue := WStrToFloat(PosGrid.Cells[Col, Row], Error);
end;

procedure TOrderForm.PageControl1Changing(Sender: TObject;
  var AllowChange: Boolean);
begin
  AllowChange := (PageControl1.ActivePage = PositionsPage) or
    ((KnoPosGrid.Results.Count>0) and (FactoryGrid.Results.Count>0));
end;

function TOrderForm.PrintOrder (Part, Parts : integer; var StopPrint : boolean) : boolean;
var PrintCount, PosCount : integer;
    FromPos, ToPos : integer;

    Found : Variant;
    FromCol, FromRow : LongInt;
    MainClip : TClipboard      ;
    Data  : string;
    Index, I : integer;

    Error   : boolean;

    SummMode : boolean; //   
begin
{
11 
: 11*2+2 (+n+1,  n - -  )
  10 ,    5*2
 :
 2:11
 12:21
 22:31

 10*(Index-1)+2:10*Index+1
  }
  Result := false;
  PosCount := 2*Ini.ReadInteger('Options','PosCountToPage', 6);
  FromPos  := PosCount*(Part-1)+2;
  ToPos    := PosCount*Part+1;
  if FromPos > pred(PosGrid.ColCount) then begin
    Result := true;
    exit;
  end;
  if ToPos > pred(PosGrid.ColCount) then begin
    SummMode := true;
    ToPos := pred(PosGrid.ColCount);
  end;
  // 7
  // 2+2*7 = 16
  // uniq 2
  // 16+2+1 = 19 / uniq from 19 - 1 - 2 = 16
  // 1: 2  : 11
  // 2: 12 : 21
  //      ,     ,
  //     
  if FromPos > PosGrid.ColCount - UnitsIds.Count - 1 then begin
     FromPos := PosGrid.ColCount - UnitsIds.Count - 1;
     SummMode := true;
  end;
  //      ,    
  //     
  if (ToPos > PosGrid.ColCount - UnitsIds.Count - 1) and
     (ToPos < pred(PosGrid.ColCount)) then
       ToPos := PosGrid.ColCount - UnitsIds.Count - 2;
  PrintCount := ToPos - FromPos + 1; //   - 
  //  ,   !

  if (Reporter.PrintAction<>paPrint) and (Part>1) then
    StopPrint := not YesNoMsg('  ?');
  if StopPrint then exit;

  with Reporter do begin
    ClearParams;
    SetValueByName('#', NumberE.AsString);
    SetValueByName('#', OrdDateE.AsString);
    // SetValueByName('#', KnoC.Text);
    if Parts>1 then SetValueByName('#', ': '+IntToStr(Part))
    else SetValueByName('#', '');
    Template := SubReg.PathToReports+'\.xls';

    SetCursor(crHourGlass);
    if not OpenExcel then exit;
    if not PrintTitle then exit;
    MainClip := Clipboard;
    //    -   
    //         
    // clipboard
    // 1.    ?
    try
      Found := ExlRange(BlankRange).Find(What := erGridLeftColumnMark,
               lookAt := xlWhole);
      if TVarData(Found).VDispatch <> nil then FromCol := Found.Column+1;
    except
      ErrorMsg('  -    "'+erGridLeftColumnMark+'"!');
      exit;
    end;
    try
      Found := ExlRange(BlankRange).Find(What := '#',
                 lookAt := xlWhole);

      if TVarData(Found).VDispatch <> nil then FromRow := Found.Row;
    except
      ErrorMsg('  -    "#"!');
      exit;
    end;

    //   
    if FactoryGrid.Results.Count > 1 then begin
      ExlRange( ExcelRectToStr(0, FromRow+2, 0,
               FromRow+FactoryGrid.Results.Count+2)).Insert (Shift := xlDown);
        //           
        //   !
      ExlRange( ExcelRectToStr(0, FromRow+FactoryGrid.Results.Count+3,
                            0, FromRow+FactoryGrid.Results.Count+3)).Copy;
      ExlRange( ExcelRectToStr(0, FromRow+2, 0, FromRow+FactoryGrid.Results.Count+2)).Select;
      Sheet.Paste;
    end;

    //   
    if PrintCount >= 1 then begin
      if not SummMode then begin
        ExlRange( ExcelRectToStr(FromCol+2, 0, FromCol+PrintCount-1,
                     0)).Insert (Shift := xlToRight);
        ExlRange( ExcelRectToStr(FromCol+PrintCount, FromRow,
                     FromCol+PrintCount+1, FromRow+FactoryGrid.Results.Count+3)).Copy;
        ExlRange( ExcelRectToStr(FromCol+2, FromRow, FromCol+PrintCount+1,
                     FromRow+FactoryGrid.Results.Count+3)).Select;
        Sheet.Paste;
      end
      else begin
        // 1 
        // 3 
        //  1+3-1
        if PrintCount = UnitsIds.Count + 1 then begin
          //   ,      
          ExlRange( ExcelRectToStr(FromCol+PrintCount, 0,
                       FromCol+PrintCount+UnitsIds.Count-2, 0)).Insert;
        end
        else begin
          ExlRange( ExcelRectToStr(FromCol+2, 0, FromCol+PrintCount-1,
                     0)).Insert (Shift := xlToRight);
          ExlRange( ExcelRectToStr(FromCol+PrintCount, FromRow,
                       FromCol+PrintCount+1, FromRow+FactoryGrid.Results.Count+3)).Copy;
          ExlRange( ExcelRectToStr(FromCol+2, FromRow, FromCol+PrintCount
                 -UnitsIds.Count, FromRow+FactoryGrid.Results.Count+3)).Select;
          Sheet.Paste;
        end;
      end;
    end;

    //     !
    if not SummMode then begin
      ExlRange( ExcelRectToStr(FromCol+PrintCount, 0,
                                    FromCol+PrintCount+1,0)).Copy;
      ExlRange( ExcelRectToStr(FromCol+2, 0,
                                    FromCol+PrintCount+1, 0)).Select;
      MsExcel.Selection.PasteSpecial (Paste := xlFormats, Operation := xlNone,
         SkipBlanks := False, Transpose := False);
    end
    else
     if PrintCount <> UnitsIds.Count + 1 then begin
      ExlRange( ExcelRectToStr(FromCol+PrintCount, 0,
                                    FromCol+PrintCount+1,0)).Copy;
      ExlRange( ExcelRectToStr(FromCol+2, 0,
                            FromCol+PrintCount-UnitsIds.Count, 0)).Select;
      MsExcel.Selection.PasteSpecial (Paste := xlFormats, Operation := xlNone,
         SkipBlanks := False, Transpose := False);
      ExlRange( ExcelRectToStr(FromCol+PrintCount, 0,
                                    FromCol+PrintCount,0)).Copy;
      ExlRange( ExcelRectToStr(FromCol+PrintCount-UnitsIds.Count+1, 0,
                            FromCol+PrintCount, 0)).Select;
      MsExcel.Selection.PasteSpecial (Paste := xlFormats, Operation := xlNone,
         SkipBlanks := False, Transpose := False);
     end;

    ExlCells(1,1).Select; //     

    Data  := ''; //      
    for Index := 2 to pred(PosGrid.RowCount) do begin
      Data := Data + PosGrid.Cells[0, Index]+#9+PosGrid.Cells[1, Index];
      if Index < pred(PosGrid.RowCount) then Data := Data + #13;
    end;
    SwitchToRussian;
    MainClip.Clear;
    MainClip.Open;
    MainClip.SetTextBuf(PChar(Data));
    MainClip.Close;
    try
      ExlCells(FromRow+2, FromCol).Select;
      Sheet.Paste;
    except
      RestoreCursor;
      MainClip.Clear;
      exit;
    end;
    MainClip.Clear;
    RestoreLayout;

    for I := FromPos to ToPos-1 do begin
      Data := '';
      for Index := 2 to pred(PosGrid.RowCount) do begin
        Data := Data + PosGrid.Cells[I  , Index]+#9+
                       PosGrid.Cells[I+1, Index];
        if Index < pred(PosGrid.RowCount) then Data := Data + #13;
      end;
      MainClip.Clear;
      MainClip.Open;
      MainClip.SetTextBuf(PChar(Data));
      MainClip.Close;
      try
        if PrintCount <> UnitsIds.Count + 1 then begin
          ExlCells(FromRow+2, FromCol+(I-(Part-1)*PosCount)).Select;
          Sheet.Paste;
          ExlCells(FromRow, FromCol+(I-(Part-1)*PosCount)).Value :=
             PosGrid.Cells[I, 0];
          if SummMode and (I >= PosGrid.ColCount - UnitsIds.Count - 1)
             and (I < pred(PosGrid.ColCount)) then
            ExlCells(FromRow+1, FromCol+(I-(Part-1)*PosCount)).Value :=
               PosGrid.Cells[I, 1];
        end
        else begin
          ExlCells(FromRow+2, FromCol+2+I-FromPos).Select;
          Sheet.Paste;
          if I = FromPos then ExlCells(FromRow, FromCol+2).Value :=
             PosGrid.Cells[I, 0];
          ExlCells(FromRow+1, FromCol+2+I-FromPos).Value :=
             PosGrid.Cells[I, 1];
        end;
      except
        RestoreCursor;
        MainClip.Clear;
        exit;
      end;
    end;
    MainClip.Clear;

    RestoreCursor;
    case PrintAction of
      paPreview : begin
        ShowExcel;
        Sheet.PrintPreview;
      end;
      paPrint : Sheet.PrintOut (Copies := PrintCopies, PrintToFile := PrintToFile);
      paShowExcel : ShowExcel;
    end;
  end;
  Result := true;
end;

procedure TOrderForm.PrintBClick(Sender: TObject);
var aPrintAction : TsohoExcelPrintAction;
    aPrintToFile : boolean;
    aCopies : integer;
    Parts : integer;
    Index : integer;
    StopPrint : boolean;
begin
  with Reporter do begin
    aPrintAction := PrintAction;
    aPrintToFile := PrintToFile;
    aCopies := PrintCopies;
    if PrintDialog then
      if not GetExcelReportOptions(aPrintAction, aPrintToFile, aCopies) then exit;
    PrintAction := aPrintAction;
    PrintToFile := aPrintToFile;
    PrintCopies := aCopies;

    Parts := trunc((KnoPosGrid.Results.Count+1)/Ini.ReadInteger('Options','PosCountToPage', 6));
    StopPrint := false;
    for Index := 0 to Parts do begin
      if not PrintOrder(Index+1, Parts+1, StopPrint) then begin
        ErrorMsg(' !   .');
        exit;
      end;
      if StopPrint then exit;
    end;
  end;
end;

procedure TOrderForm.KnoPosQFilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
var Id, IdKNO, IdUnits : LongInt;
begin
  with DataSet do begin
    IdKno   := FieldByName('IdKNO').AsInteger;
    Id      := FieldByName('Id').AsInteger;
    IdUnits := FieldByName('IdUnits').AsInteger;
  end;
  Accept := KnoGrid.Results.IndexOf(IdKNO)<>-1;
  if (not Accept) and (KnoPosGrid.Results.IndexOf(Id)<>-1) then
    RemovePosition(Id, IdUnits);
end;

initialization
  OrderForm := nil;
end.
