unit Main;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     AppEvent, SoRxTls, RXShell, Menus, SohoRgst, SoTVCont, DB, DBTables,
     Grids, SoDBGrid, SoCtrls, SohoBtns, SoExlRep, StdCtrls, ExtCtrls,
     RXCtrls, SohoTree, SoCtmRgs, SoCtmTre;

const ImportGroupCreate : boolean = false;
      ReportsDir : string = '';

type
  TMainForm = class(TForm)
    App: TsohoApplication;
    RxTrayIcon1: TRxTrayIcon;
    TrayMenu: TPopupMenu;
    ProjectsM: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    ProjectList: TsohoTVContainer;
    Reg: TsohoRegister;
    N5: TMenuItem;
    N6: TMenuItem;
    N7: TMenuItem;
    Reporter: TsohoExcelReporter;
    AddBtns: TPanel;
    PrintB: TsohoBitBtn;
    RxSpeedButton1: TRxSpeedButton;
    RxSpeedButton2: TRxSpeedButton;
    ImportMenuM: TPopupMenu;
    ExportMenuM: TPopupMenu;
    BugList1: TMenuItem;
    N8: TMenuItem;
    N9: TMenuItem;
    SaveD: TSaveDialog;
    N10: TMenuItem;
    N11: TMenuItem;
    N12: TMenuItem;
    N13: TMenuItem;
    N14: TMenuItem;
    OpenD: TOpenDialog;
    PopupMenu1: TPopupMenu;
    Test1: TMenuItem;
    ShowClosedM: TMenuItem;
    procedure AppHide(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure N4Click(Sender: TObject);
    procedure ProjectsMClick(Sender: TObject);
    procedure ProjectListCardCreate(Sender: TObject);
    procedure ProjectListFormCreate(Sender: TObject);
    procedure ProjectListFormDestroy(Sender: TObject);
    procedure ReporterBeforePrint(Sender: TObject; var Allow: Boolean);
    procedure ReporterGetText(Sender: TObject; Field: TField; Column,
      Row: Integer; var Text: String);
    procedure PrintBClick(Sender: TObject);
    procedure ExportClicked(Sender: TObject);
    procedure ImportClicked(Sender: TObject);
    procedure ProjectListNewGrp(Sender: TObject; NewGroup: TsohoTreeNode;
      var Allow: Boolean);
    procedure Test1Click(Sender: TObject);
    procedure ProjectListCustomNewGrp(Sender: TObject; GroupId, GroupHash,
      ParentId: Integer; var GroupName: String; var Allow: Boolean);
    procedure ProjectListCustomEditGrp(Sender: TObject; GroupId, GroupHash,
      ParentId: Integer; var GroupName: String; var Allow: Boolean);
    procedure ShowClosedMClick(Sender: TObject);
    procedure ProjectListGridDblClick(Sender: TObject);
    procedure ProjectListGetCellProps(Sender: TObject; Field: TField;
      AFont: TFont; var Background: TColor);
    procedure ProjectListFilter(Sender: TObject; DataSet: TDataSet;
      var Visible: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
    FCount : LongInt;
    FNewGroup: TsohoTreeNode;
    ShowClosedMenuIndex : LongInt;
    procedure FindShowClosedMenu;
  end;

var MainForm: TMainForm;

implementation
uses SohoRec, SoUtils, SoDBRtn, SoDate, SoUnit, ProjForm, SoCtmRec, DBGrids,
     SoBDERtn;

{$R *.DFM}

procedure TMainForm.AppHide(Sender: TObject);
begin
   Application.Minimize;
   ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
   ShowClosedMenuIndex := -1;
   Reg.Activate;
   // ProjectList.DataBaseName := Reg.PathToData;
   // ProjectList.FolderDir    := Reg.PathToHome;
   ReportsDir               := Reg.PathToReports;
   with Reporter do begin
     Options := [erGridFields, erListFields];
     PrintFields.Add('ORDNUMBER');
   end;
   AppHide(Self);
   ProjectsMClick(ProjectsM);
end;

procedure TMainForm.N4Click(Sender: TObject);
begin
   Close;
end;

procedure TMainForm.ProjectsMClick(Sender: TObject);
begin
   { Projects tree view }
   ProjectList.Execute(false, true, false);
end;

procedure TMainForm.ProjectListCardCreate(Sender: TObject);
var EndDate : TsohoEdit;
begin
  //       - !
  if ProjectList.TreeViewForm.RecordCard.Mode<>rmInsert then exit;
  EndDate := TsohoEdit(ProjectList.TreeViewForm.RecordCard.DialogForm.
             GetControlByFieldName('DaEndDate'));
  if EndDate=nil then exit;
  EndDate.Text := '';
end;

procedure TMainForm.FindShowClosedMenu;
var Index : LongInt;
    Item : TMenuItem;
begin
  with ProjectList.TreeViewForm.TreeView.DBGrid.PopupMenu do begin
    for Index := 0 to pred(Items.Count) do begin
      Item := Items[Index];
      if Item.Caption = ShowClosedM.Caption then
        ShowClosedMenuIndex := Index;
    end;
  end;
end;

procedure TMainForm.ProjectListFormCreate(Sender: TObject);
var BtnParent : TWinControl;
begin
   {     TreeView }
   ShowClosedM.Checked := ProjectList.TreeViewForm.Folder.Ini.ReadBool('Main',
     'ShowClosedTasks', false);
   if ShowClosedMenuIndex = -1 then FindShowClosedMenu;
   ProjectList.TreeView.DBGrid.PopupMenu.Items[ShowClosedMenuIndex].Checked := ShowClosedM.Checked;
   BtnParent      := ProjectList.TreeView.BtnsPanel;
   with AddBtns do begin
     Parent  := BtnParent;
     Top     := ProjectList.TreeView.ChildCheckBox.Top + 60;
     Width   := BtnParent.Width;
     Left    := 0;
   end;
   with ProjectList.TreeView do begin
      DBGrid.Options := DBGrid.Options - [dgIndicator];
      DBGrid.RowHeightPercent := 300;
      DBGrid.WordWrap := true;
      Folder.LoadDBGridColumns;
   end;
end;

procedure TMainForm.ProjectListFormDestroy(Sender: TObject);
begin
   {      }
   AddBtns.Parent := Self;
   ProjectList.TreeViewForm.Folder.Ini.WriteBool('Main',
      'ShowClosedTasks', ShowClosedM.Checked);
end;

procedure TMainForm.ReporterBeforePrint(Sender: TObject;
  var Allow: Boolean);
begin
   FCount := 0;
end;

procedure TMainForm.ReporterGetText(Sender: TObject; Field: TField; Column,
  Row: Integer; var Text: String);
begin
   if Field is TBooleanField then begin
      if Field.AsBoolean then Text := ''
      else Text := '';
   end;
   if AnsiUpperCase(Field.FieldName) = 'ORDNUMBER' then begin
     inc(FCount);
     Text := IntToStr(FCount);
   end;
end;

procedure TMainForm.PrintBClick(Sender: TObject);
var ProjId   : LongInt;
    Leaf     : TTreeLeaf;
    ProjName : string;
    ProjDate : string;
    ProjCust : string;
    Query    : TQuery;
begin
  //      
  Leaf   := ProjectList.TreeViewForm.TreeView.GetLeafRecord(ProjectList.TreeViewForm.TreeView.GroupOutLine.SelectedItem);
  ProjId := Leaf.ItemId;
  ProjName := ' ';
  ProjCust := '';
  ProjDate := DateToStr(Date);
  Query    := ProjectList.TreeViewForm.GroupsQ;
  if LocaleById(TDataSet(Query),'Id', ProjId) then begin
    ProjName := Query.FieldByName('Name').AsString;
    ProjCust := Query.FieldByName('ChCustomer').AsString;
    ProjDate := Query.FieldByName('DaBegDate').AsString;
  end;
  with Reporter do begin
     Folder   := ProjectList.TreeViewForm.TreeView.Folder;
     SetValueByName('#',ProjName);
     SetValueByName('#_', ProjDate);
     SetValueByName('#', ProjCust);
     PrintDocument(ReportsDir+'.xls', '');
  end;
end;

procedure TMainForm.ExportClicked(Sender: TObject);
const RepBools : array [boolean] of string = (' |','*|');
var GrpsQ, ItmsQ : TDataSet;
    ExprtFile    : TextFile;
    s            : string;
    ExportRecord : boolean;
begin
  //     ...
  if not SaveD.Execute then exit;
  ProjectList.TreeViewForm.TreeView.LocateOnGroup;
  GrpsQ := ProjectList.GroupsQuery;
  ItmsQ := ProjectList.ItemsQuery;
  try
    try
      AssignFile(ExprtFile, SaveD.FileName);
      Rewrite(ExprtFile);
      case (Sender as TMenuItem).Tag of
        1 : s := ' ';
        2 : s := '   ';
        3 : s := '  ';
        4 : s := '  ';
        5 : s := '   ';
      end;
      writeln(ExprtFile, s);
      writeln(ExprtFile, Chars('-',3+11+11+201));
      writeln(ExprtFile, '  : ', GrpsQ.FieldByName('Name').AsString);
      writeln(ExprtFile, ': ', GrpsQ.FieldByName('ChCustomer').AsString);
      writeln(ExprtFile, '   : ', GrpsQ.FieldByName('DaBegDate').AsString);
      writeln(ExprtFile, ': ', GrpsQ.FieldByName('DaEndDate').AsString);
      writeln(ExprtFile, '  : ', BoolToStr(GrpsQ.FieldByName('LoDone').AsBoolean));
      writeln(ExprtFile, ' : ', BoolToStr(GrpsQ.FieldByName('LoPayed').AsBoolean));
      writeln(ExprtFile, '   : ', GrpsQ.FieldByName('FlSumm').AsString);
      writeln(ExprtFile, '.  : ', GrpsQ.FieldByName('FlCashSumm').AsString);
      writeln(ExprtFile, Chars('-',11+11+11+201));
      writeln(ExprtFile, '| |B|W|T|O| ',
                         AlignStr('' ,taCenter, ' ', 9),'| ',
                         AlignStr('',taCenter, ' ', 9),'| ',
                         AlignStr('',taCenter, ' ', 199),'|');
      writeln(ExprtFile, Chars('-',11+11+11+201));
      with ItmsQ do begin
        First;
        while not Eof do begin
          case (Sender as TMenuItem).Tag of
            1 : ExportRecord := FieldByName('LoBugT').AsBoolean;
            2 : ExportRecord := FieldByName('LoBugT').AsBoolean or
                                FieldByName('LoWishT').AsBoolean;
            3 : ExportRecord := FieldByName('LoDone').AsBoolean;
            4 : ExportRecord := true;
            5 : ExportRecord := ProjectList.TreeViewForm.TreeView.Results.IndexOf(
                FieldByName('Id').AsInteger)<>-1;
          end;
          if ExportRecord then begin
             s := '|'+RepBools[FieldByName('LoDone').AsBoolean] +
                  RepBools[FieldByName('LoBugT').AsBoolean] +
                  RepBools[FieldByName('LoWishT').AsBoolean] +
                  RepBools[FieldByName('LoTaskT').AsBoolean] +
                  RepBools[FieldByName('LoOtherT').AsBoolean] +
                  AlignStr(FieldByName('DaBegDate').AsString, taCenter, ' ', 10) + '|' +
                  AlignStr(FieldByName('DaEndDate').AsString, taCenter, ' ', 10) + '|' +
                  AlignStr(FieldByName('Name').AsString,    taLeftJustify, ' ', 200)+ '|';
             writeln(ExprtFile, s);
          end;
          Next;
        end;
        writeln(ExprtFile, Chars('-',11+11+11+201));
        First;
      end;
    except ErrorMsg('  !');
    end;
  finally
    CloseFile(ExprtFile);
  end;
end;

procedure TMainForm.ImportClicked(Sender: TObject);
var
    ImprtFile      : TextFile;
    s              : string;
    PName, Customer: string;
    BDate, EDate   : TDateTime;
    Done, Payed    : boolean;
    Summ, CashSumm : Double;

    TName : string;
    TDone : boolean;
    TBTime, TETime : TDateTime;
    BT, TT, WT, OT : boolean;

    ProjId   : LongInt;
    Leaf     : TTreeLeaf;

    ItemHash : LongInt;
    Error : boolean;

    procedure ReadProjectTitle;
    begin
      readln(ImprtFile, s); //  
      readln(ImprtFile, s); //  

      SetDivisers([':']);
      readln(ImprtFile, s); //   
      PName := GetSomeWords(s,2,2);
      readln(ImprtFile, s); //  
      Customer := GetSomeWords(s,2,2);
      readln(ImprtFile, s); //   
      BDate := WStrToDate(GetSomeWords(s,2,2));
      readln(ImprtFile, s); //   
      EDate := WStrToDate(GetSomeWords(s,2,2));
      readln(ImprtFile, s); //    
      Done := StrToBool(GetSomeWords(s,2,2));
      readln(ImprtFile, s); //    
      Payed := StrToBool(GetSomeWords(s,2,2));
      readln(ImprtFile, s); //  
      Summ := WStrToFloat(GetSomeWords(s,2,2), Error);
      readln(ImprtFile, s); //   
      CashSumm := WStrToFloat(GetSomeWords(s,2,2), Error);
      readln(ImprtFile, s); //  
    end;

begin

  //     ...
  if not OpenD.Execute then exit;
  try
      AssignFile(ImprtFile, OpenD.FileName);
      Reset(ImprtFile);
  except ErrorMsg('  !');
      CloseFile(ImprtFile);
  end;
  if (Sender as TMenuItem).Tag = 1 then begin //  
      ProjectList.TreeViewForm.TreeView.GroupOutLine.SelectedItem := 1;
      ReadProjectTitle;
      ImportGroupCreate := true;
      if not ProjectList.TreeViewForm.TreeView.QuietAddGroup(PName) then begin
        ImportGroupCreate := false;
        exit;
      end;
      ImportGroupCreate := false;
      //    ,   FNewGroup     
      // .      ,  
      //    
      ProjId := FNewGroup.GroupId;
      if not ExecuteSQL(SingleRegister.PathToSQL+'Updproj.sql', Reg.PathToData, [
        IntToStr(ProjId),
        DateToSQLDate(BDate),
        DateToSQLDate(EDate),
        BoolToStr(Done),
        BoolToStr(Payed),
        SoUtils.FloatToSQLFloat(Summ),
        SoUtils.FloatToSQLFloat(CashSumm)
      ]) then begin
        ErrorMsg('  !');
        CloseFile(ImprtFile);
        exit;
      end;
  end
  else begin
     //   GroupId   
     Leaf   := ProjectList.TreeViewForm.TreeView.GetLeafRecord(ProjectList.TreeViewForm.TreeView.GroupOutLine.SelectedItem);
     ProjId := Leaf.ItemId;
     ReadProjectTitle;
  end;
  //      
  SetDivisers(['|']);
  //  Hash  
  ProjectList.ItemsQuery.Last;
  ItemHash := ProjectList.ItemsQuery.FieldByName('Hash').AsInteger;
  readln(ImprtFile, s); //  
  readln(ImprtFile, s); //  
  repeat
    readln(ImprtFile, s);
    if s[1]='|' then begin
      TDone  := GetSomeWords(s, 1, 1)='*';
      BT     := GetSomeWords(s, 2, 2)='*';
      TT     := GetSomeWords(s, 3, 3)='*';
      WT     := GetSomeWords(s, 4, 4)='*';
      OT     := GetSomeWords(s, 5, 5)='*';
      TBTime := WStrToDate(GetSomeWords(s, 6, 6));
      TETime := WStrToDate(GetSomeWords(s, 7, 7));
      TNAme  := GetSomeWords(s, 8, 8);
      ItemHash := ItemHash + 100;
      if not ExecuteSQL(SingleRegister.PathToSQL+'addtask.sql', Reg.PathToData, [
        IntToStr(GetNewId(ProjectList.ItemsPreffix)),
        IntToStr(ProjId),
        IntToStr(ItemHash),
        TName,
        DateToSQLDate(TBTime),
        DateToSQLDate(TETime),
        BoolToStr(TDone),
        BoolToStr(BT),
        BoolToStr(WT),
        BoolToStr(TT),
        BoolToStr(OT)
      ]) then begin
        ErrorMsg('  : '+TName+' !');
        CloseFile(ImprtFile);
        exit;
      end;
    end;
  until Eof(ImprtFile);
  CloseFile(ImprtFile);
  ProjectList.TreeViewForm.TreeView.RefreshItems;
end;

procedure TMainForm.ProjectListNewGrp(Sender: TObject;
  NewGroup: TsohoTreeNode; var Allow: Boolean);
begin
  if not ImportGroupCreate then begin
    Allow := true;
    exit;
  end;
  //       
  FNewGroup := NewGroup;
  Allow := true;
end;

procedure TMainForm.Test1Click(Sender: TObject);
begin
  if ExecuteSQL(SingleRegister.PathToSQL+'closetsk.sql', Reg.PathToData, [
     ProjectList.ItemsQuery.FieldByName('Id').AsString,
     DateToSQLDate(Date)
     ]) then ProjectList.TreeViewForm.TreeView.RefreshItems;
end;

procedure TMainForm.ProjectListCustomNewGrp(Sender: TObject; GroupId,
  GroupHash, ParentId: Integer; var GroupName: String; var Allow: Boolean);
var aName, aCustomer, aComment : string;
    aBegDate, aEndDate : TDateTime;
    aDone, aPayed : boolean;
    aIdCustomer : LongInt;
begin
  Allow := false;
  aIdCustomer := -1;
  aName := GroupName;
  if NewProject(aName, aCustomer, aIdCustomer, aBegDate,
     aEndDate, aDone, aPayed, aComment) then
      Allow := ExecuteSQL(SingleRegister.PathToSQL+'newproj.sql', Reg.PathToData, [
      IntToStr(GroupId),
      IntToStr(ParentId),
      IntToStr(GroupHash),
      aName,
      aCustomer,
      IntToStr(aIdCustomer),
      DateToSQLDate(aBegDate),
      DateToSQLDate(aEndDate),
      BoolToStr(aDone),
      BoolToStr(aPayed),
      aComment
    ]);
   if Allow then begin
     FNewGroup.GroupId   := GroupId;
     FNewGroup.GroupName := aName;
     GroupName := aName;
   end;
end;

procedure TMainForm.ProjectListCustomEditGrp(Sender: TObject; GroupId,
  GroupHash, ParentId: Integer; var GroupName: String; var Allow: Boolean);
var aName, aCustomer, aComment : string;
    aBegDate, aEndDate : TDateTime;
    aDone, aPayed : boolean;
    aIdCustomer : LongInt;
    Query : TDataSet;
begin
  Allow := false;
  Query := ProjectList.GroupsQuery;
  if not ProjectList.TreeViewForm.TreeView.LocateOnGroup then exit;
  with Query do begin
    aName       := FieldByName('Name').AsString;
    aCustomer   := FieldByName('ChCustomer').AsString;
    aIdCustomer := FieldByName('IdCustomer').AsInteger;
    aBegDate    := FieldByName('DaBegDate').AsDateTime;
    aEndDate    := FieldByName('DaEndDate').AsDateTime;
    aDone       := FieldByName('LoDone').AsBoolean;
    aPayed      := FieldByName('LoPayed').AsBoolean;
    aComment    := FieldByName('ChComment').AsString;
  end;
  if EditProject(aName, aCustomer, aIdCustomer, aBegDate,
     aEndDate, aDone, aPayed, aComment) then
      Allow := ExecuteSQL(SingleRegister.PathToSQL+'updproj.sql', Reg.PathToData, [
      aName,
      aCustomer,
      IntToStr(aIdCustomer),
      DateToSQLDate(aBegDate),
      DateToSQLDate(aEndDate),
      BoolToStr(aDone),
      BoolToStr(aPayed),
      aComment,
      IntToStr(GroupId)
    ]);
   if Allow then GroupName := aName;
end;

procedure TMainForm.ShowClosedMClick(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
  ShowClosedM.Checked := (Sender as TMenuItem).Checked;
  ProjectList.TreeView.RefreshItems;
end;

procedure TMainForm.ProjectListGridDblClick(Sender: TObject);
begin
   if not ProjectList.EditOnDblClick then
    ProjectList.TreeViewForm.SelectMClick(ProjectList.TreeViewForm.SelectM);
end;

procedure TMainForm.ProjectListGetCellProps(Sender: TObject; Field: TField;
  AFont: TFont; var Background: TColor);
var FName : string;
begin
  if Field <> nil then
  with (Sender as TsohoDBGrid), Canvas, DataSource.DataSet do begin
     FName := AnsiUpperCase(Field.FieldName);
     if (FName='DABEGDATE') or (FName='DAENDDATE') then aFont.Color := clBlue;
     if (FieldByName('DAENDDATE').AsDateTime>0) and
        (FieldByName('DAENDDATE').AsDateTime<Date) then aFont.Color := clRed;
     if FieldByName('LoDone').AsBoolean then aFont.Color := clSilver;
  end;
end;

procedure TMainForm.ProjectListFilter(Sender: TObject; DataSet: TDataSet;
  var Visible: Boolean);
begin
  Visible := Visible and (ShowClosedM.Checked or
             (not DataSet.FieldByName('LoDone').AsBoolean));
end;

end.
