{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{          Copyright (c) 1996-1997 AllexSoft            }
{                   Written by VSM                      }
{                                                       }
{                   SOHO Components                     }
{                                                       }
{*******************************************************}
{
     TsohoCheckBox
}
unit SoChkBox;

interface
uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
     StdCtrls, Menus;

{$I SOHOLIB.INC}

type

  TsohoCheckBoxType = (btCheckBox, btRadioButton);

  TsohoCustomCheckBox = class(TCustomControl)
  private
    { Private declarations }
    FCheckStyle : TsohoCheckBoxType;
    FChecked : boolean;
    FCaption : string;
    FInRect : boolean;
    FToggleKey: TShortCut;
    FAlignment: TAlignment;
    FOnChange: TNotifyEvent;
  protected
    { Protected declarations }
    procedure SetStyle (Value : TsohoCheckBoxType);
    procedure SetChecked (Value : boolean);
    procedure SetCaption (Value : string);
    procedure SetAlignment(Value: TAlignment);
    procedure SetState(Value: TCheckBoxState);
    function GetState : TCheckBoxState;

    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMEnter(var message: TCMEnter); message CM_ENTER;
    procedure CMExit(var message: TCMExit); message CM_EXIT;
    procedure CreateParams(var Params: TCreateParams);override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure KeyDown(var Key: Word; Shift: TShiftState);override;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure Changed;virtual;

    property Style: TsohoCheckBoxType read FCheckStyle write SetStyle;
    property Checked: boolean read FChecked write SetChecked;
    property Caption: string read FCaption write SetCaption;
    property ToggleKey: TShortCut read FToggleKey write FToggleKey;
    property State: TCheckBoxState read GetState write SetState;
    property Alignment: TAlignment read FAlignment write SetAlignment default taRightJustify;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;

  public
    { Public declarations }
    procedure Paint;override;
    constructor Create(aOwner : TComponent);override;
    procedure ToggleCheck;
  published
    property TabStop default true;
    property TabOrder;
  end;

  TsohoCheckBox = class(TsohoCustomCheckBox)
  published
    property Alignment;
    property Caption;
    property Checked;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property State default cbUnchecked;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnChange;
    property Style default btCheckBox;
  end;

function GetGroupIndex (GroupBox: TGroupBox): integer;
procedure SetGroupIndex (GroupBox: TGroupBox; Index: integer);

implementation
uses SoUtils, ExtCtrls;

{$R SOCHKBOX.R32}

var
   CheckBmps,
   RadioBmps : TBitmap;

function GetGroupIndex (GroupBox: TGroupBox): integer;
var Index : integer;
begin
  Result := 0;
  for Index := 0 to pred(GroupBox.ControlCount) do
    if GroupBox.Controls[Index] is TsohoCustomCheckBox then
      if TsohoCustomCheckBox(GroupBox.Controls[Index]).Checked then begin
        Result := Index;
        exit;
      end;
end;

procedure SetGroupIndex (GroupBox: TGroupBox; Index: integer);
begin
    if GroupBox.Controls[Index] is TsohoCustomCheckBox then
      TsohoCustomCheckBox(GroupBox.Controls[Index]).Checked := true;
end;

procedure TsohoCustomCheckBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMDialogChar(var Message: TCMDialogChar);
begin
  if Enabled and IsAccel(Message.CharCode, Caption) then begin
     SetFocus;
     ToggleCheck;
     Message.Result := 1;
  end;
end;

procedure TsohoCustomCheckBox.SetAlignment(Value: TAlignment);
begin
  if FAlignment = Value then exit;
  FAlignment := Value;
  Invalidate;
end;

procedure TsohoCustomCheckBox.SetState(Value: TCheckBoxState);
begin
  case Value of    //
    cbUnchecked : begin
      Enabled := true;
      Checked := false;
    end;
    cbChecked   : begin
      Enabled := true;
      Checked := true;
    end;
    cbGrayed    : begin
      Enabled := false;
      Checked := true;
    end;
  end;    // case
end;

function TsohoCustomCheckBox.GetState : TCheckBoxState;
begin
  if not Checked then Result := cbUnchecked
  else
    if Enabled then Result := cbChecked
    else Result := cbGrayed;
end;

procedure TsohoCustomCheckBox.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then begin
    if TabStop and CanFocus then SetFocus;
    ToggleCheck;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TsohoCustomCheckBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if Enabled and (FToggleKey = ShortCut(Key, Shift)) then begin
    ToggleCheck;
    Key := 0;
  end;
end;

procedure TsohoCustomCheckBox.ToggleCheck;
begin
  Checked := not Checked;
end;

procedure TsohoCustomCheckBox.SetStyle (Value : TsohoCheckBoxType);
begin
  if FCheckStyle = Value then exit;
  FCheckStyle := Value;
  Invalidate;
end;

procedure TsohoCustomCheckBox.Changed;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TsohoCustomCheckBox.SetChecked (Value : boolean);

  procedure TurnSiblingsOff;
  var Index: LongInt;
      Sibling: TControl;
  begin
    if Parent <> nil then
      with Parent do
        for Index := 0 to pred(ControlCount) do begin
          Sibling := Controls[Index];
          if (Sibling <> Self) and (Sibling is TsohoCheckBox) and
             (TsohoCheckBox(Sibling).Style = btRadioButton) then
            TsohoCheckBox(Sibling).Checked := false;
        end;
  end;

begin
  if FChecked = Value then exit;
  FChecked := Value;
  if Value and (FCheckStyle = btRadioButton) then TurnSiblingsOff;
  Changed;
  Invalidate;
end;

procedure TsohoCustomCheckBox.SetCaption (Value : string);
begin
   if FCaption = Value then exit;
   FCaption := Value;
   Invalidate;
end;

procedure TsohoCustomCheckBox.CMEnabledChanged (var Message: TMessage);
begin
   inherited;
   Invalidate;
end;

procedure TsohoCustomCheckBox.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  FInRect := true;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMMouseLeave (var Message: TMessage);
begin
  inherited;
  FInRect := false;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMEnter(var message: TCMEnter);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMExit(var message: TCMExit);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomCheckBox.Paint;
var TmpR, R : TRect;
    {x, }y : LongInt;
    I1, I2 : integer;
begin
  with Canvas do begin
    Brush.Color := Self.Color;
    Font := Self.Font; 
    R := ClientRect;
    FillRect(R);
    if FCheckStyle = btCheckBox then
       Y := VertCenter(R, CheckBmps.Height)
    else
       Y := VertCenter(R, RadioBmps.Height);
    I1 := 0;
    if FChecked then I1 := 1;
    if Enabled then begin
       if FInRect or (Screen.ActiveControl = Self) then I2 := 0
       else I2 := 1;
    end
    else I2 := 2;
    if FCheckStyle = btCheckBox then begin
       TmpR := Bounds(0, y, 13, 13);
       Frame3D(Canvas, TmpR, clBtnShadow, clBtnHighlight, 1);
       CopyRect( Bounds(1, y+1, 11, 11), CheckBmps.Canvas,
                 Bounds(I2 * 22 + I1 * 11, 0, 11, 11))
    end
    else
       CopyRect( Bounds(0, y, 12, 12), RadioBmps.Canvas,
                 Bounds(I2 * 24 + I1 * 12, 0, 12, 12));
    R.Left := 18;
    Y := VertCenter(R, TextHeight(FCaption));
    R.Top := Y;
    R.Bottom := R.Top + TextHeight(FCaption);
    R.Right := R.Left + TextWidth(FCaption);
    TextInRect(Canvas, FCaption, R, FAlignment);
    if Screen.ActiveControl = Self then begin
      R.Left := R.Left - 4;
      R.Right := R.Right + 2;
      if R.Right > Left + Width then R.Right := Left + Width; 
        
      R.Bottom := R.Bottom + 1;
      DrawFocusRect(R);
    end;
  end;
end;

procedure TsohoCustomCheckBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do begin
    WindowClass.Style := WindowClass.Style or CS_HREDRAW or CS_VREDRAW;
  end;
end;

constructor TsohoCustomCheckBox.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  ControlStyle := [csClickEvents, csSetCaption, csCaptureMouse,
    csOpaque, csDoubleClicks];
  FCheckStyle := btCheckBox;
  FChecked := false;
  FCaption := '';
  FInRect := false;
  Height := 14;
  Width := 75;
  TabStop := True;
  FToggleKey := vk_Space;
  FAlignment := taLeftJustify;
end;

initialization
   CheckBmps := TBitmap.Create;
   CheckBmps.LoadFromResourceName(hInstance, 'FLATCHECKS');
   RadioBmps := TBitmap.Create;
   RadioBmps.LoadFromResourceName(hInstance, 'FLATRADIO');
finalization
   CheckBmps.Free;
   RadioBmps.Free;
end.
