{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{          Copyright (c) 1996-1997 AllexSoft            }
{                   Written by VSM                      }
{                                                       }
{                   SOHO Components                     }
{                                                       }
{*******************************************************}
{
     TsohoRegionForm,   
    .
}
unit SoCmplFm;

{$I SOHOLIB.INC}

interface
uses Controls, WinTypes, WinProcs, Graphics, Classes;

type

  {    TsohoRegionFigure:
    rtRectangle - 
    rtRoundRect -    
    rtEllipse   - 
  }
  TRegionType = (rtRectangle, rtRoundRect, rtEllipse);

  {      ,  
      .     
          }
  TsohoRegionFigure = class(TGraphicControl)
  private
    FRegionType: TRegionType;
    procedure SetRegionType(Value: TRegionType);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    {   Parent }
    property Align;
    {   }
    property RegionType: TRegionType read FRegionType write SetRegionType default rtRectangle;
  end;

  {     :
    cmAnd  -   
    cmCopy -     
    cmDiff -  ,     
    cmOr   -   
    cmXor  -   ,    
  }
  TCombineMode = (cmAnd, cmCopy, cmDiff, cmOr, cmXor);

  {     :
    roIncludeMenu    -       
    roIncludeCaption -      
  }
  TRegionsOption = (roIncludeMenu, roIncludeCaption);
  {      }
  TRegionsOptions = set of TRegionsOption;

  {      ( TForm) 
    .        TsohoRegionFigure,
        design-time.     
      API  -   AddRegion   
    OnBeforeApply.     ,  bitmap  
     CreateRgnFromBitmap.
  }
  TsohoRegionForm = class(TComponent)
  private
    FMainMode     : TCombineMode;
    FRegionsMode  : TCombineMode;
    FOptions      : TRegionsOptions;
    FOnFormCreate : TNotifyEvent;
    FBeforeApply  : TNotifyEvent;
    FFormRegion   : HRGN;
    FRegions      : TList;
    procedure DoFormCreate   (Sender : TObject);
  protected
    procedure SetFormRegions;
    procedure Loaded;override;
  public
    {    Bitmap. ClientX, ClientY -  
      ,  ,    Bitmap.
      TransparentColor - ,    ,  ,
      ,     }
    function CreateRgnFromBitmap (ClientX, ClientY : integer;
             Bitmap : TBitmap; TransparentColor : TColor) : HRGN;
    {      ,   
         }
    procedure AddRegion (Value : HRGN);virtual;
    {     }
    function RegionsCount : LongInt;
    {   TsohoRegionFigure   }
    function AddRegionFigure : boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    {    .  ,   cmAnd 
           ,     .  ,
        cmXor     ,      }
    property MainMode    : TCombineMode    read FMainMode write FMainMode default cmAnd;
    {        }
    property RegionsMode : TCombineMode    read FRegionsMode write FRegionsMode default cmOr;
    {     :     }
    property Options     : TRegionsOptions read FOptions write FOptions default [];
    {        .
               }
    property OnBeforeApply : TNotifyEvent read FBeforeApply write FBeforeApply;
  end;

implementation
uses SoUtils, Forms, SoShell;

{ TsohoRegionFigure }
constructor TsohoRegionFigure.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FRegionType := rtRectangle;
  Width := 65;
  Height := 65;
end;

procedure TsohoRegionFigure.Paint;
var X, Y, W, H, S: Integer;
begin
  if not (csDesigning in ComponentState) then exit;
  with Canvas do begin
    with Pen do begin
      Color := clBlue;
      Width := 1;
      Style := psDashDot;
    end;
    Brush.Style := bsClear;
    X := Pen.Width div 2;
    Y := X;
    W := Width - Pen.Width + 1;
    H := Height - Pen.Width + 1;
    if Pen.Width = 0 then begin
      Dec(W);
      Dec(H);
    end;
    if W < H then S := W else S := H;
    case FRegionType of
      rtRectangle : Rectangle(X, Y, X + W, Y + H);
      rtRoundRect : RoundRect(X, Y, X + W, Y + H, S div 4, S div 4);
      rtEllipse   : Ellipse(X, Y, X + W, Y + H);
    end;
  end;
end;

procedure TsohoRegionFigure.SetRegionType(Value: TRegionType);
begin
  if FRegionType <> Value then begin
    FRegionType := Value;
    Invalidate;
  end;
end;

{ TsohoRegionForm }
procedure TsohoRegionForm.DoFormCreate   (Sender : TObject);
begin
  SetFormRegions;
  if Assigned(FOnFormCreate) then FOnFormCreate(Sender);
end;

procedure TsohoRegionForm.AddRegion (Value : HRGN);
begin
   FRegions.Add(pointer(Value));
end;

procedure TsohoRegionForm.SetFormRegions;
const
  Modes : array [TCombineMode] of integer = (RGN_AND, RGN_COPY, RGN_DIFF,
                                             RGN_OR, RGN_XOR);

var Index : LongInt;
    Figure : TsohoRegionFigure;
    NewRegion : HRGN;
    XOffset, YOffset : LongInt;
    Form : TForm;
    ResultRegion : HRGN;
begin
  Form := GetOwnerForm(Self);
  YOffset := GetCaptionHeight(Form)+GetMainMenuHeight(Form) +
             GetBorderHeight(Form);
  XOffset := trunc((Form.Width-Form.ClientWidth)/2);

  for Index := 0 to pred(Form.ControlCount) do
    if Form.Controls[Index] is TsohoRegionFigure then begin
      Figure := Form.Controls[Index] as TsohoRegionFigure;
      with Figure do
       case RegionType of
        rtEllipse   : NewRegion := CreateEllipticRgn(XOffset+Left, YOffset+Top,
                      XOffset+Left+Width, YOffset+Top+Height);
        rtRectangle : NewRegion := CreateRectRgn(XOffset+Left, YOffset+Top,
                      XOffset+Left+Width, YOffset+Top+Height);
        else
        {rtRoundRect : }NewRegion := CreateRoundRectRgn(XOffset+Left, YOffset+Top,
                      XOffset+Left+Width, YOffset+Top+Height, 20, 20);
       end;
      AddRegion(NewRegion);
    end;
  if (roIncludeCaption in FOptions) and
     (Form.BorderStyle<>bsNone) then begin
     NewRegion := CreateRectRgn(0, 0, Form.Width, GetCaptionHeight(Form));
     AddRegion(NewRegion);
  end;
  if (roIncludeMenu in FOptions) and Assigned(Form.Menu) then begin
     NewRegion := CreateRectRgn(0, GetCaptionHeight(Form)+GetBorderHeight(Form),
       Form.Width, GetCaptionHeight(Form)+GetBorderHeight(Form)+GetMainMenuHeight(Form));
     AddRegion(NewRegion);
  end;
  if Assigned(FBeforeApply) then FBeforeApply(Self);
  if FRegions.Count>0 then begin
    ResultRegion := HRGN(FRegions[0]);
    for Index := 1 to pred(FRegions.Count) do
      CombineRgn(ResultRegion, ResultRegion, HRGN(FRegions[Index]),Modes[FRegionsMode]);
    FFormRegion := CreateRectRgn(0,0,Form.Width, Form.Height);
    {ResultRegion := }CombineRgn(FFormRegion, FFormRegion, ResultRegion,
                    Modes[FMainMode]);
    SetWindowRgn(Form.Handle, FFormRegion, true);
    for Index := 0 to pred(FRegions.Count) do DeleteObject(HRGN(FRegions[Index]));
  end;
end;

procedure TsohoRegionForm.Loaded;
var //Index : LongInt;
    Form  : TForm;
begin
  inherited Loaded;
  if csDesigning in ComponentState then exit;
  Form := GetOwnerForm(Self);
  FOnFormCreate := Form.OnCreate;
  Form.OnCreate := DoFormCreate;
end;

function TsohoRegionForm.RegionsCount : LongInt;
begin
  Result := FRegions.Count;
end;

function TsohoRegionForm.AddRegionFigure : boolean;
var NewRegion : TsohoRegionFigure;
    Form : TForm;
begin
  Result := false;
  Form := GetOwnerForm(Self);
  try
    NewRegion := TsohoRegionFigure.Create(Form);
    NewRegion.Name := GetUniqueName(NewRegion, Form);
    NewRegion.Parent := Form;
    Result := true;
  except
  end;
end;

function TsohoRegionForm.CreateRgnFromBitmap (ClientX, ClientY : integer;
             Bitmap : TBitmap; TransparentColor : TColor) : HRGN;
var StX,EndX : LongInt;
    Y : LongInt;
    NewR : HRGN;
    XOffset, YOffset : LongInt;
    Form : TForm;
begin
   Form := GetOwnerForm(Self);
   YOffset := GetCaptionHeight(Form)+GetMainMenuHeight(Form) +
              GetBorderHeight(Form);
   XOffset := trunc((Form.Width-Form.ClientWidth)/2);

   Result  := CreateRectRgn(ClientX+XOffset, ClientY+YOffset,
                            ClientX+XOffset, ClientY+YOffset); // 
   for Y := 0 to pred(Bitmap.Height) do begin
     StX := 0;
     while StX < Bitmap.Width-1 do begin
       while (StX < Bitmap.Width-1) and
             (Bitmap.Canvas.Pixels[StX,Y]=TransparentColor) do inc(StX);
       if StX < Bitmap.Width-1 then begin
         EndX := StX;
         while (EndX < Bitmap.Width-1) and
               (Bitmap.Canvas.Pixels[EndX,Y]<>TransparentColor) do inc(EndX);
         NewR := CreateRectRgn(ClientX+XOffset+StX, ClientY+YOffset+Y,
                               ClientX+XOffset+EndX, ClientY+YOffset+Y+1);
         CombineRgn(Result, Result, NewR, RGN_OR);
         DeleteObject(NewR);
         StX := EndX + 1;
       end;
     end;
   end;
end;

constructor TsohoRegionForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMainMode     := cmAnd;
  FRegionsMode  := cmOr;
  FOptions      := [];
  FOnFormCreate := nil;
  FRegions      := TList.Create;
end;

destructor TsohoRegionForm.Destroy;
//var Index : LongInt;
begin
  if FRegions.Count>0 then begin
    DeleteObject(FFormRegion);
    FRegions.Free;
  end;
  if Assigned(FOnFormCreate) and not (csDestroying in ComponentState)
    then GetOwnerForm(Self).OnCreate := FOnFormCreate;
  inherited Destroy;
end;

initialization
  RegisterClasses([TsohoRegionFigure]);
end.
