{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{          Copyright (c) 1996-1997 AllexSoft            }
{                   Written by VSM                      }
{                                                       }
{                   SOHO Components                     }
{                                                       }
{*******************************************************}
{
     TsohoEnhTreeView -   TTreeView 
   .     
      (ExtendedTreeView)
}
unit SoEnhTreeView;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, Classes, Graphics, ComCtrls, Controls, SoCommCtrl
     {$IFDEF Delphi4}
     ,ImgList
     {$ENDIF};

type

  {$IFDEF Delphi4}
  TTVPrePaintEvent = procedure(Sender: TObject;
    Node: TTreeNode;
    dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT;
    var clrText, clrTextBk: COLORREF) of object;
  {$ELSE}
  TTVPrePaintEvent = procedure(Sender: TObject;
    Node: TTreeNode;
    dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT;
    var clrText, clrTextBk: TColor) of object;
  {$ENDIF}

  TTVPaintEvent = procedure(Sender: TObject;
    Node: TTreeNode;
    Canvas: TCanvas;
    Rect: TRect;
    dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT) of object;

  TsohoEnhCheckState = (csChecked, csUnchecked, csPartChecked);

  TsohoNodeClickNotify = procedure (Sender : TObject; Node : TTreeNode) of object;

  TsohoEnhTreeView = class(TTreeView)
  private
    FTrackSelect: Boolean;
    FOnPrePaint: TTVPrePaintEvent;
    FOnPaint: TTVPaintEvent;
    FOnPostPaint: TTVPaintEvent;
    FOnNodeClick : TsohoNodeClickNotify;
    FStates : TImageList;
    FToolTips: Boolean;
    FShowChecks : boolean;
    FAutoChecks : boolean;
    // FShowArrows : boolean;
    FOnCheckClick : TsohoNodeClickNotify;
    // FOnGetImageIndex: TTVExpandedEvent;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CNNotify(var Msg: TWMNotify); message CN_NOTIFY;
    procedure Loaded;override;
    procedure SetCheckForChilds (aItem : TTreeNode; aChecked : boolean);
    procedure SetCheckForParent (aItem : TTreeNode);
    procedure WMNotify(var Message: TWMNotify); message WM_NOTIFY;
    procedure SetTrackSelect (Value : boolean);
    procedure SetChecks (Value : boolean);
    procedure SetToolTips (Value: boolean);
    function GetToolTips: boolean;
    {procedure SetArrows (Value : boolean);
    procedure GetImageIndex(Sender: TObject; Node: TTreeNode);virtual;}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;override;
    function GetNodeState (Node : TTreeNode) : TsohoEnhCheckState;
    procedure SetNodeState (Node : TTreeNode; Value : TsohoEnhCheckState);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure KeyPress(var Key: Char);override;
    function  IsNodeBold(Node: TTreeNode): Boolean;
    procedure MakeNodeBold(Node: TTreeNode; Value: Boolean);
  published
    // property OnGetImageIndex: TTVExpandedEvent read FOnGetImageIndex write FOnGetImageIndex;
    property AutoChecks : boolean read FAutoChecks write FAutoChecks default true;
    property ShowChecks : boolean read FShowChecks write SetChecks default false;
    // property ShowArrows : boolean read FShowArrows write SetArrows default true;
    property ToolTips: boolean read GetToolTips write SetToolTips;
    property TrackSelect: Boolean read FTrackSelect write SetTrackSelect default True;
    property OnPrePaint: TTVPrePaintEvent read FOnPrePaint write FOnPrePaint;
    property OnPaint: TTVPaintEvent read FOnPaint write FOnPaint;
    property OnPostPaint: TTVPaintEvent read FOnPostPaint write FOnPostPaint;
    property OnNodeClick : TsohoNodeClickNotify read FOnNodeClick write FOnNodeClick;
    property OnCheckNode : TsohoNodeClickNotify read FOnCheckClick write FOnCheckClick;
  end;

function InitCommonControlsEx(dwICC: DWORD): Boolean;

implementation
uses SysUtils, CommCtrl;

{$R SOENHTREEVIEW.R32}

const
  cctrl = 'comctl32.dll';

type
  TInitCommonControlsEx = record
    dwSize: DWORD;
    dwICC: DWORD;
  end;

function InitCommonControlsEx_(var TInitCommonControlsRec: TInitCommonControlsEx): Boolean;
  stdcall external cctrl name 'InitCommonControlsEx';

function InitCommonControlsEx;
var
  InitCommonControlsExRec: TInitCommonControlsEx;
begin
  InitCommonControlsExRec.dwSize := SizeOf(InitCommonControlsExRec);
  InitCommonControlsExRec.dwICC := dwICC;
  Result := InitCommonControlsEx_(InitCommonControlsExRec);
end;

const  TVS_NOTOOLTIPS = $0080;

function TsohoEnhTreeView.GetToolTips: boolean;
begin
  FToolTips := (GetWindowLong(Handle, GWL_STYLE) and TVS_NOTOOLTIPS) <> 0;
  Result := FToolTips;
end;

procedure TsohoEnhTreeView.SetToolTips (Value: boolean);
var State : longint;
begin
  if FToolTips = Value then exit;
  FToolTips := Value;
  State := GetWindowLong(Handle, GWL_STYLE);
  if FToolTips then State := State or TVS_NOTOOLTIPS
  else State := State xor TVS_NOTOOLTIPS;
  SetWindowLong(Handle, GWL_STYLE, State);
end;

procedure TsohoEnhTreeView.WMNotify(var Message: TWMNotify);
begin
 inherited;
 // if FToolTips then inherited;
end;

procedure TsohoEnhTreeView.SetTrackSelect (Value : boolean);
begin
  if FTrackSelect = Value then exit;
  FTrackSelect := Value;
  RecreateWnd;
end;

{
procedure TsohoEnhTreeView.GetImageIndex(Sender: TObject; Node: TTreeNode);
begin
  if FShowArrows then begin
    if Node.HasChildren then begin
      if not Node.Expanded then Node.StateIndex := 1
      else Node.StateIndex := 2;
    end
    else Node.StateIndex := 0;
  end
  else if Assigned(FOnGetImageIndex) then FOnGetImageIndex(Sender, Node);
end;

procedure TsohoEnhTreeView.SetArrows (Value : boolean);
begin
  if FShowArrows = Value then exit;
  FShowArrows := Value;
  ShowChecks := false;
  if FShowArrows then begin
    with FStates do begin
      Width := 12;
      Height := 12;
      ResourceLoad(rtBitmap, 'SOHOARROWSREEVIEW', clWhite);
    end;    // with
    StateImages := FStates;
  end
  else StateImages := nil;
end;
}

procedure TsohoEnhTreeView.SetChecks (Value : boolean);
begin
  if FShowChecks = Value then exit;
  FShowChecks := Value;
  // ShowArrows := false;
  if csDesigning in ComponentState then exit;
  if FShowChecks then begin
    {with FStates do begin
      Width := 16;
      Height := 16;
      ResourceLoad(rtBitmap, 'SOHOCHECKTREEVIEW', clWhite);
    end;    // with}
    StateImages := FStates;
  end
  else StateImages := nil;
end;

procedure TsohoEnhTreeView.KeyPress(var Key: Char);
var Item : TTreeNode;
begin
  if FShowChecks and (ReadOnly) and (Key = #32) then begin
    Key := #00;
    Item := Selected;
    if Assigned(FOnCheckClick) then FOnCheckClick(Self, Item);
    if FAutoChecks then begin
      SetCheckForChilds(Item, Item.StateIndex <= 1);
      SetCheckForParent(Item);
    end;
    Invalidate;
  end
  else inherited KeyPress(Key);
end;

procedure TsohoEnhTreeView.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ItemRect : TRect;
    Pnt : TPoint;
begin
  if Button <> mbLeft then inherited MouseUp(Button, Shift, X, Y)
  else begin
    inherited MouseUp(Button, Shift, X, Y);
    if not Assigned(Selected) then exit;
    ItemRect := Selected.DisplayRect(true);
    Pnt.X := X; Pnt.Y := Y;
    if PtInRect(ItemRect, Pnt) then
      if Assigned(FOnNodeClick) then FOnNodeClick(Self, Selected);
  end;
end;

procedure TsohoEnhTreeView.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Item : TTreeNode;
    ItemRect : TRect;
    Offset : integer;
begin
  if Button <> mbLeft then begin
    if RightClickSelect then begin
      Item := GetNodeAt(X, Y);
      if Item <> nil then Selected := Item;
    end;
    inherited MouseDown(Button, Shift, X, Y);
  end
  else begin
    if not FShowChecks then begin
      inherited MouseDown(Button, Shift, X, Y);
      exit;
    end;

    Item := GetNodeAt(X, Y);
    if (Item = nil) or (Item <> Selected) then exit;
    ItemRect := Item.DisplayRect(true);
    Offset := 14;
    if (Images <> nil) and (Item.ImageIndex <> -1) then
      Offset := Offset + Images.Width;
    if (X >= ItemRect.Left - Offset) and
       (X <= ItemRect.Left - Offset + 14) then begin
      if Assigned(FOnCheckClick) then FOnCheckClick(Self, Item);
      if FAutoChecks then begin
        SetCheckForChilds(Item, Item.StateIndex <= 1);
        SetCheckForParent(Item);
      end;
      Invalidate;
    end
    else inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TsohoEnhTreeView.SetCheckForChilds (aItem : TTreeNode; aChecked : boolean);
var Index : LongInt;
begin
  if aChecked then aItem.StateIndex := 2
  else aItem.StateIndex := 1;
  for Index := 0 to pred(aItem.Count) do
    SetCheckForChilds(aItem.Item[Index], aChecked);
end;

procedure TsohoEnhTreeView.SetCheckForParent (aItem : TTreeNode);
var Index, CheckCount, Almost : LongInt;
    Tmp : TTreeNode;
begin
  Tmp := aItem.Parent;
  if Tmp = nil then exit;
  CheckCount := 0;
  Almost := 0;
  for Index := 0 to pred(Tmp.Count) do begin
    if Tmp.Item[Index].StateIndex = 2 then inc(CheckCount);
    if Tmp.Item[Index].StateIndex = 3 then inc(Almost);
  end;
  if CheckCount = Tmp.Count then Tmp.StateIndex := 2
  else
     if (CheckCount = 0) and (Almost = 0) then Tmp.StateIndex := 1
     else Tmp.StateIndex := 3;
  if Tmp.Parent <> nil then SetCheckForParent (Tmp);
end;

procedure TsohoEnhTreeView.Loaded;
begin
  inherited Loaded;
  if csDesigning in ComponentState then exit;
  {if FShowArrows then begin
    FShowArrows := false;
    ShowArrows := true;
  end;}
  FStates.ResourceLoad(rtBitmap, 'SOHOCHECKTREEVIEW', clWhite);
end;

function TsohoEnhTreeView.GetNodeState (Node : TTreeNode) : TsohoEnhCheckState;
begin
  if (Node.StateIndex < 2) or (Node.StateIndex > 3) then Result := csUnchecked
  else
    if Node.StateIndex = 2 then Result := csChecked
    else Result := csPartChecked;
end;

procedure TsohoEnhTreeView.SetNodeState (Node : TTreeNode; Value : TsohoEnhCheckState);
begin
  case Value of    //
    csChecked      : Node.StateIndex := 2;
    csUnchecked    : Node.StateIndex := 1;
    csPartChecked  : Node.StateIndex := 3;
  end;    // case
  // if Assigned(FOnCheckClick) then FOnCheckClick
end;

constructor TsohoEnhTreeView.Create;
begin
  inherited Create(AOwner);
  FTrackSelect := True;
  FShowChecks := false;
  // FShowArrows := true;
  FAutoChecks := true;
  FStates := TImageList.Create(Self);
  // inherited OnGetImageIndex := GetImageIndex;
end;

destructor TsohoEnhTreeView.Destroy;
begin
  FStates.Free;
  inherited Destroy;
end;

procedure TsohoEnhTreeView.CreateParams;
const
  TrackSelectStyles: array[Boolean] of DWORD = (0, TVS_TRACKSELECT);
begin
  if not InitCommonControlsEx(ICC_TREEVIEW_CLASSES) then
    raise Exception.Create('Probably your COMCTL32.DLL is obsolete. Get a new one');

  inherited CreateParams(Params);
  with Params do
    Style := Style or TrackSelectStyles[FTrackSelect];
end;

procedure TsohoEnhTreeView.CNNotify;

  procedure CallPaintEvent(PaintEvent: TTVPaintEvent);
  var
    Canvas: TCanvas;
  begin
    Canvas := TCanvas.Create;
    with TNMTVCustomDraw(Pointer(Msg.NMHdr)^), nmcd do
    try
      Canvas.Handle := hdc;
      Canvas.Brush := Self.Brush;
      Canvas.Font := Font;
      PaintEvent(Self,
        TTreeNode(lItemlParam),
        Canvas,
        rc,
        dwItemSpec,
        uItemState);
    finally
      Canvas.Free;
    end;
  end;

begin
  inherited;

  with Msg, TNMTVCustomDraw(Pointer(NMHdr)^), nmcd do
  case Hdr.Code of
    NM_CUSTOMDRAW:
    begin
      case dwDrawStage of
        (*
           CDDS_PREPAINT is at the beginning of the paint cycle. You
           implement custom draw by returning the proper value. In this
           case, we're requesting item-specific notifications.
        *)
        CDDS_PREPAINT:
          // Request pre-paint notifications for each item.
          if Assigned(FOnPrePaint) or Assigned(FOnPaint) or Assigned(FOnPostPaint) then
            Result := CDRF_NOTIFYITEMDRAW;
        (*
          CDDS_ITEMPREPAINT is sent when the control is about to paint an
          item. We're getting this because we returned CDRF_NOTIFYITEMDRAW
          in response to CDDS_PREPAINT before.
        *)
        CDDS_ITEMPREPAINT:
        begin
          if Assigned(FOnPrePaint) then
          begin
            (*
              To change the text and background colors in a list view
              control, set the clrText and clrTextBk members of the
              NMLVCUSTOMDRAW structure to the desired color.

              This differs from most other controls that support
              CustomDraw. To change the text and background colors for
              the others, call SetTextColor and SetBkColor on the provided HDC.
            *)
            FOnPrePaint(Self,
              TTreeNode(lItemlParam),
              dwItemSpec,
              uItemState,
              clrText, clrTextBk);
            Result := Result or CDRF_NEWFONT;
          end;
          if Assigned(FOnPaint) then // we wouldn't get here if haven't asked this
                                         // we will not ask this until Assigned(FOnPostPaint
                                         // BUT I leave this check. Just in case
          begin
            CallPaintEvent(FOnPaint);
            Result := Result or CDRF_SKIPDEFAULT;
          end;
          if Assigned(FOnPostPaint) then
            Result := Result or CDRF_NOTIFYPOSTPAINT;
        end;
        CDDS_ITEMPOSTPAINT:
          if Assigned(FOnPostPaint) then // we wouldn't get here if haven't asked this
                                         // we will not ask this until Assigned(FOnPostPaint
                                         // BUT I leave this check. Just in case
            CallPaintEvent(FOnPostPaint);
      end;
    end;
  end;
end;

function TsohoEnhTreeView.IsNodeBold(Node: TTreeNode): Boolean;
var Item: TTVItem;
begin
  Result := False;
  if not Assigned(Node) then Exit;

  with Item do begin
    mask := TVIF_STATE;
    hItem := Node.ItemId;
    if TreeView_GetItem(Node.Handle, Item) then
      Result := (State and TVIS_BOLD) <> 0;
  end;
end;

procedure TsohoEnhTreeView.MakeNodeBold(Node: TTreeNode; Value: Boolean);
var Item: TTVItem;
    Template: Integer;
begin
  if not Assigned(Node) then Exit;

  if Value then Template := -1
  else Template := 0;

  with Item do begin
    mask := TVIF_STATE;
    hItem := Node.ItemId;
    StateMask := TVIS_BOLD;
    State := StateMask and Template;
  end;
  TreeView_SetItem(Node.Handle, Item);
end;


end.
