unit SoGrCntr;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms;


const

  DefWidth  : LongInt = 15;
  DefHeight : LongInt = 15;
  LeftField : LongInt = 40;
  TopField  : LongInt = 40;

type

  TsohoGraphControl = class;

  TsohoGraphNode = class
  private
    { Private declarations }
    FOwner : TsohoGraphControl;
    FLeft, FTop : LongInt;
    FCaption : string;
    FColor : TColor;
    FLinks : TList;
  protected
    { Protected declarations }
    procedure SetColor(Value : TColor);
    procedure SetCaption(Value : string);
    procedure SetLeft(Value : LongInt);
    procedure SetTop(Value : LongInt);
  public
    { Public declarations }
    constructor Create(aOwner : TsohoGraphControl);
    destructor Destroy;override;
    property Links : TList read FLinks;
    property Left : LongInt read FLeft write SetLeft;
    property Top : LongInt read FTop write SetTop;
    property Caption : string read FCaption write SetCaption;
    property Color : TColor read FColor write SetColor;
  end;

  TsohoGraphControl = class(TCustomControl)
  private
    { Private declarations }
    FNodes : TList;
    FScale : integer;
    FXOffset, FYOffset : integer;
    FNodeWidth, FNodeHeight : integer;
    FRecalcCoeff : Double;
    FDrawCanvas : TCanvas;
  protected
    { Protected declarations }
    procedure SetScale(Value : integer);
    procedure DrawNodes;
    procedure DrawLinks;
  public
    { Public declarations }
    constructor Create(aOwner : TComponent);override;
    destructor Destroy;override;
    procedure Paint;override;
    procedure PrepareNodes;
    function AddNode(const Caption : string) : TsohoGraphNode;
    procedure Clear;
    property Nodes : TList read FNodes;
  published
    { Published declarations }
    property Scale : integer read FScale write SetScale default 100;
    property XOffset : Integer read FXOffset write FXOffset default 80;
    property YOffset : integer read FYOffset write FYOffset default 50;
    property NodeWidth : Integer read FNodeWidth write FNodeWidth default 15;
    property NodeHeight : Integer read FNodeHeight write FNodeHeight default 15;
    property Align;
    property Color;
    property Font;
  end;

implementation
uses SoUtils;

{ TsohoGraphNode }
procedure TsohoGraphNode.SetColor(Value : TColor);
begin
  if FColor = Value then exit;
  FColor := Value;
  FOwner.Invalidate;
end;

procedure TsohoGraphNode.SetCaption(Value : string);
begin
  if FCaption = Value then exit;
  FCaption := Value;
  FOwner.Invalidate;
end;

procedure TsohoGraphNode.SetLeft(Value : LongInt);
begin
  if FLeft = Value then exit;
  FLeft := Value;
  // FOwner.Invalidate;
end;

procedure TsohoGraphNode.SetTop(Value : LongInt);
begin
  if FTop = Value then exit;
  FTop := Value;
  // FOwner.Invalidate;
end;

constructor TsohoGraphNode.Create(aOwner : TsohoGraphControl);
begin
   FOwner := aOwner;
   FLeft := 0;
   FTop := 0;
   FCaption := '';
   FColor := clWhite;
   FLinks := TList.Create;
end;

destructor TsohoGraphNode.Destroy;
begin
  FLinks.Free;
  inherited Destroy;
end;

{ TsohoGraphControl }
procedure TsohoGraphControl.Clear;
var Index : LongInt;
begin
  for Index := 0 to pred(FNodes.Count) do TsohoGraphNode(FNodes[Index]).Free;
  FNodes.Clear;
end;

procedure TsohoGraphControl.SetScale(Value : integer);
begin
  if (FScale = Value) or (FScale <= 10) then exit;
  FRecalcCoeff := Value / FScale;
  FScale := Value;
  Repaint;
end;

constructor TsohoGraphControl.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  FNodes := TList.Create;
  FScale := 100;
  FXOffset := 80;
  FYOffset := 50;
  FNodeWidth := DefWidth;
  FNodeHeight := DefHeight;
end;

destructor TsohoGraphControl.Destroy;
begin
  FNodes.Free;
  inherited Destroy;
end;

procedure TsohoGraphControl.DrawNodes;
var aRect : TRect;
    Index : LongInt;
    x, y : LongInt;
begin
  with FDrawCanvas do begin
    for Index := 0 to pred(FNodes.Count) do begin    // Iterate
      with TsohoGraphNode(FNodes[Index]) do begin
        Brush.Style := bsSolid;
        Brush.Color := Color;
        aRect := Rect(Left, Top, Left + FNodeWidth, Top + FNodeHeight);
        FillRect(aRect);
        Pen.Color := clBlack;
        DrawFocusRect(aRect);
        Font := Self.Font;
        x := TextWidth(Caption);
        if x > FNodeWidth then x := Left - trunc((x - FNodeWidth)/2)
        else x := Left + trunc((FNodeWidth - x)/2);
        Brush.Style := bsClear;
        TextOut(X, Top + FNodeHeight + 1, Caption);
      end;    // with
    end;    // for
  end;
end;

procedure TsohoGraphControl.DrawLinks;
var aRect : TRect;
    J, Index : LongInt;
    aW, aH : integer;
    //   y = kx + b
    // k = (y2-y1)/(x2-x1) = dy/dx
    // b = y1-kx1
    x1,y1, x2,y2, ab, db, dx, dy, dx1, dy1 : double;
begin
  aW := NodeWidth div 2;
  aH := NodeHeight div 2;
  with FDrawCanvas do begin
    for Index := 0 to pred(FNodes.Count) do begin    // Iterate
      with TsohoGraphNode(FNodes[Index]) do begin
        for J := 0 to pred(FLinks.Count) do begin
          x1 := Left+aW;
          y1 := Top+aH;
          x2 := TsohoGraphNode(FLinks[J]).Left+aW;
          y2 := TsohoGraphNode(FLinks[J]).Top+aH;
          MoveTo(round(x1), round(y1));
          LineTo(round(x2), round(y2));
          {if x1 > x2 then SwapDouble(x1, x2);
          if y1 > y2 then SwapDouble(y1, y2);}
          dx := x2-x1; dy:= y2-y1;
          ab:= sqrt(dx*dx + dy*dy);
          if ab>0 then begin
          db:= ab-12;
          dx1 := dx*db/ab;
          dy1 := dy*db/ab;
          x1 := x1 + dx1;
          y1 := y1 + dy1;
          Ellipse(round(x1-3),round(y1-3),round(x1+3),round(y1+3));
          Brush.Color := clBlack;
          Brush.Style := bsSolid;
          FloodFill(round(x1), round(y1), clBlack, fsBorder);
          end;
        end;
      end;    // with
    end;    // for
  end;    // with
end;

procedure TsohoGraphControl.Paint;
var aRect : TRect;
    Index, J : LongInt;
    MemBmp : TBitmap;
begin
  try
    MemBmp := TBitmap.Create;
    MemBmp.Width := Width;
    MemBmp.Height := Height;
    FDrawCanvas := MemBmp.Canvas;
    with FDrawCanvas do begin
      aRect := ClientRect;
      Brush.Color := Self.Color;
      FillRect(aRect);
      Pen.Color := clBlack;
      DrawLinks;
      DrawNodes;
    end;    // with
  finally // wrap up
    BitBlt(Canvas.Handle, 0, 0, Width, Height, MemBmp.Canvas.Handle, 0, 0, SRCCOPY);
    MemBmp.Free;
  end;    // try/finally
end;

procedure TsohoGraphControl.PrepareNodes;
var Cols : LongInt;
    I, X, Y, Index : LongInt;
begin
   Cols := trunc(sqrt(FNodes.Count));
   x:= 5; Y := 5;
   I := 0;
   for Index := 0 to pred(FNodes.Count) do begin    // Iterate
      with TsohoGraphNode(FNodes[Index]) do begin
        Top := Y + TopField;
        Left := X + LeftField;
      end;    // with
      x:= x + FnodeWidth + FXOffset;
      inc(I);
      if I > Cols then begin
       I := 0;
       x:=5;
       Y:= Y + FNodeHeight + FYOffset;
     end;
   end;    // for
   Width := LeftField + 2*Cols * (FnodeWidth + FXOffset) + LeftField;
   Height := TopField + 2*Cols * (FnodeHeight + FYOffset) + TopField;
   Repaint;
end;

function TsohoGraphControl.AddNode(const Caption : string) : TsohoGraphNode;
begin
  Result := TsohoGraphNode.Create(Self);
  Result.Caption := Caption;
  FNodes.Add(Result);
end;

end.
