{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{          Copyright (c) 1996-1997 AllexSoft            }
{               Written by VSM, ICM                     }
{                                                       }
{                   SOHO Components                     }
{                                                       }
{*******************************************************}
unit SoListView;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, ComCtrls,
     StdCtrls;

const

  DRAWTEXTEX_FLAGS = DT_NOPREFIX or DT_SINGLELINE or DT_VCENTER or
     DT_END_ELLIPSIS;
  DRAWTEXTEX_ALIGNMENT: array[TAlignment] of UINT = (DT_LEFT, DT_RIGHT,
     DT_CENTER);
  WM_OWNERDRAWCOLUMNS = WM_USER + 143;

type

  TsohoLVPrePaintEvent = procedure(Sender: TObject;
    Node: TListItem;
    dwItemSpec: Longint;
    uItemState: UINT;
    var clrText, clrTextBk: TColor) of object;

  TsohoLVPaintEvent = procedure(Sender: TObject;
    Node: TListItem;
    Canvas: TCanvas;
    Rect: TRect;
    dwItemSpec: Longint;
    uItemState: UINT) of object;

  TsohoLVColumnNotify = procedure (Sender: TObject; Column: TListColumn) of object;

  {   :       }
  TColumnSortType  = (lvsString, lvsInteger);

  {    :
   lvmManual -    .   
                 OnColumnClick, ,
                     
               ColumnSortType
   lvmJustify -        
                 :     , 
                  ,     }

  TSelectSortMetod = (lvmManual, lvmJustify);

  TsohoListView = class(TListView)
  private
    { Private declarations }
    FOnPrePaint: TsohoLVPrePaintEvent;
    FOnPaint: TsohoLVPaintEvent;
    FOnPostPaint: TsohoLVPaintEvent;
    FBeforeColResize,
    FAfterColResize: TsohoLVColumnNotify;

    FAutoWidth: boolean;
    FStriply: boolean;
    FStriplyColor: TColor;

    FCanColResize: boolean;
    FAutoSort: boolean;
    FLastSortIndex: integer;
    FAscOrder: boolean;
    FUpBmp,
    FDownBmp : TBitmap;

    FColumnSortType : TColumnSortType;
    FSelectSortMetod: TSelectSortMetod;

    procedure SetAutoWidth(Value: boolean);
    procedure SetHotItem (Index: integer);
    function GetHotItem : integer;
    procedure SetHotCursor (Value: TCursor);
    function GetHotCursor: TCursor;
    procedure SetStriply (Value: boolean);
    procedure SetStriplyColor (Value: TColor);
    procedure SetUpBmp (Value: TBitmap);
    procedure SetDownBmp (Value: TBitmap);
    Procedure SetColumnSortType(Value : TColumnSortType);
    Procedure SetSortMetod(Value : TSelectSortMetod);
  protected
    { Protected declarations }
    FCanvas: TCanvas;
    FHeaderHandle: HWND;
    // procedure CreateParams(var Params: TCreateParams);override;
    procedure CNNotify(var Msg: TWMNotify); message CN_NOTIFY;
    procedure DrawStriply (Item : TListItem; var BkColor : TColor);virtual;
    procedure ColClick(Column: TListColumn); override;
    procedure WMNotify(var Message: TWMNotify); message WM_Notify;
    procedure DoBeforeColResize(Column: TListColumn); virtual;
    procedure DoAfterColResize(Column: TListColumn); virtual;
    procedure WMSIZE(var Message: TWMSize); message WM_SIZE;
    procedure WMDrawHeader(var Message: TWMDrawItem); message WM_DRAWITEM;
    procedure WMOwnerDrawColumns(var Message: TMessage); message WM_OWNERDRAWCOLUMNS;
    procedure RecalcWidth;
    // procedure HeaderWndProc(var Message: TMessage);override;
    procedure SetColumnSort(ColumnIndex: integer);
    procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure InvalidateColumnHeader(Index: integer); virtual;
    procedure ProcessDrawHeaderMsg(Index: Integer; Rect: TRect;
       State: TOwnerDrawState; var DefaultDrawing: boolean); virtual;
    procedure DrawHeader(var Canvas: TCanvas; Index: Integer; var Rect: TRect;
       Selected: boolean; var DefaultDrawing: boolean); virtual;
    procedure DefaultDrawHeader(var Canvas: TCanvas; Index: Integer;
       var Rect: TRect; Selected: boolean); virtual;
    // procedure SetOnDrawHeader(Value: TLVHDrawItemEvent); virtual;
    procedure SetColumnsOwnerDrawFlag(OwnerDrawn: boolean); virtual;
    procedure CreateSortBmps; virtual;
    procedure HeaderBmpChanged (Sender: TObject);
  public
    { Public declarations }
    procedure Loaded;override;
    constructor Create(aOwner: TComponent);override;
    destructor Destroy;override;
    property SortedColumn: integer read FLastSortIndex write SetColumnSort;
    property HeaderHandle: HWnd read FHeaderHandle;
  published
    { Published declarations }
    property OnPrePaint: TsohoLVPrePaintEvent read FOnPrePaint write FOnPrePaint;
    property OnPaint: TsohoLVPaintEvent read FOnPaint write FOnPaint;
    property OnPostPaint: TsohoLVPaintEvent read FOnPostPaint write FOnPostPaint;
    property BeforeColResize: TsohoLVColumnNotify read FBeforeColResize write FBeforeColResize;
    property AfterColResize: TsohoLVColumnNotify read FAfterColResize write FAfterColResize;
    property AutoWidth: boolean read FAutoWidth write SetAutoWidth default false;
    property AutoSort: boolean read FAutoSort write FAutoSort default true;
    property HotItem: integer read GetHotItem write SetHotItem;
    property HotCursor: TCursor read GetHotCursor write SetHotCursor default crDefault;
    property Striply: boolean read FStriply write SetStriply default false;
    property StriplyColor: TColor read FStriplyColor write SetStriplyColor default $00E8E8E8;
    property CanColResize: boolean read FCanColResize write FCanColResize default true;
    property SortUpBmp: TBitmap read FUpBmp write SetUpBmp;
    property SortDownBmp: TBitmap read FDownBmp write SetDownBmp;
    property ColumnSortType : TColumnSortType read FColumnSortType write SetColumnSortType default lvsString;
    property SelectSortMetod : TSelectSortMetod read FSelectSortMetod write SetSortMetod default lvmManual;
    property Align;
    property BorderStyle;
    property Color;
    property ColumnClick;
    property OnClick;
    property OnDblClick;
    property Columns;
    property Ctl3D;
    property DragMode;
    property ReadOnly default False;
    property Enabled;
    property Font;
    property GridLines;
    property HideSelection;
    property HotTrack;
    property IconOptions;
    property Items;
    property Checkboxes;
    property AllocBy;
    property MultiSelect;
    property RowSelect;
    property OnChange;
    property OnChanging;
    property OnColumnClick;
    property OnCompare;
    property OnDeletion;
    property OnEdited;
    property OnEditing;
    property OnEnter;
    property OnExit;
    property OnInsert;
    property OnDragDrop;
    property OnDragOver;
    property DragCursor;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property ShowColumnHeaders;
    property SortType;
    property TabOrder;
    property TabStop default True;
    property ViewStyle;
    property Visible;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property LargeImages;
    property SmallImages;
    property StateImages;
  end;

implementation
uses CommCtrl, SoCommCtrl;

type

  {$HINTS OFF}
  THackScreen = class(TComponent)
  private
    FFonts: TStrings;
    FImes: TStrings;
    FDefaultIme: string;
    FDefaultKbLayout: HKL;
    FPixelsPerInch: Integer;
    FCursor: TCursor;
    FCursorCount: Integer;
  end;
  {$HINTS ON}

function SohoLVAscDefaultSorting(Item1, Item2: TListItem; lParam: Integer): Integer; stdcall;
var s1,s2: string;
begin
  if lParam > 0 then begin
    if lParam > Item1.SubItems.Count then s1:= ''
    else s1:= Item1.SubItems[lParam - 1];
    if lParam > Item2.SubItems.Count then s1:= ''
    else s2:= Item2.SubItems[lParam - 1];
  end
  else begin
    s1:= Item1.Caption;
    s2:= Item2.Caption;
  end;
  Result := lstrcmp(PChar(s1), PChar(s2));
end;

function SohoLVDescDefaultSorting(Item1, Item2: TListItem; lParam: Integer): Integer; stdcall;
var s1,s2: string;
begin
  if lParam > 0 then begin
    if lParam > Item1.SubItems.Count then s1:= ''
    else s1:= Item1.SubItems[lParam - 1];
    if lParam > Item2.SubItems.Count then s1:= ''
    else s2:= Item2.SubItems[lParam - 1];
  end
  else begin
    s1:= Item1.Caption;
    s2:= Item2.Caption;
  end;
  Result := lstrcmp(PChar(s2), PChar(s1));
end;

//    
function SohoLVAscIntSorting(Item1, Item2: TListItem; lParam: Integer): Integer; stdcall;
var s1,s2 : Integer;
begin
  if lParam > 0 then begin
    if lParam > Item1.SubItems.Count then s1:= 0
    else s1:= StrToIntDef(Item1.SubItems[lParam - 1], 0);
    if lParam > Item2.SubItems.Count then s1:= 0
    else s2:= StrToIntDef(Item2.SubItems[lParam - 1], 0);
  end
  else begin
    s1:= StrToIntDef(Item1.Caption, 0);
    s2:= StrToIntDef(Item2.Caption, 0);
  end;
  Result := s1-s2;
end;

function SohoLVDescIntSorting(Item1, Item2: TListItem; lParam: Integer): Integer; stdcall;
var s1,s2 : Integer;
begin
  if lParam > 0 then begin
    if lParam > Item1.SubItems.Count then s1:= 0
    else s1:= StrToInt(Item1.SubItems[lParam - 1]);
    if lParam > Item2.SubItems.Count then s1:= 0
    else s2:= StrToInt(Item2.SubItems[lParam - 1]);
  end
  else begin
    s1:= StrToInt(Item1.Caption);
    s2:= StrToInt(Item2.Caption);
  end;
  Result := s2-s1;
end;

procedure TsohoListView.Loaded;
begin
  inherited Loaded;
  PostMessage(Handle, WM_OWNERDRAWCOLUMNS, 0, 0);
end;

procedure TsohoListView.SetColumnSort(ColumnIndex: integer);
var {Column: PsohoLVCOLUMN;}
    FNewOrder: boolean;
    i: integer;
begin
   for I := 0 to pred(Columns.Count) do
     InvalidateColumnHeader(I);
   // if FLastSortIndex <> -1 then InvalidateColumnHeader(FLastSortIndex);
   FNewOrder := FLastSortIndex <> ColumnIndex;
   if not FNewOrder then FAscOrder := not FAscOrder
   else FAscOrder := true;
   {----------    ----------}
   if FSelectSortMetod = lvmJustify then
     if Columns[ColumnIndex].Alignment = taRightJustify
       then FColumnSortType := lvsInteger
       else FColumnSortType := lvsString;
   case FColumnSortType of
   lvsString:  if FAscOrder
                 then CustomSort(@SohoLVAscDefaultSorting, ColumnIndex)
                 else CustomSort(@SohoLVDescDefaultSorting, ColumnIndex);
   lvsInteger: if FAscOrder
                 then CustomSort(@SohoLVAscIntSorting, ColumnIndex)
                 else CustomSort(@SohoLVDescIntSorting, ColumnIndex);

   end; {--- CASE ---}
   FLastSortIndex := ColumnIndex;
   InvalidateColumnHeader(ColumnIndex);
end;

procedure TsohoListView.ColClick(Column: TListColumn);
begin
  inherited;
  if FAutoSort and Assigned(Column) then SetColumnSort(Column.Index);
end;

procedure TsohoListView.RecalcWidth;
begin
  if FAutoWidth then ListView_SetColumnWidth(Handle, Columns.Count - 1,
     LVSCW_AUTOSIZE_USEHEADER);
end;

procedure TsohoListView.MouseMove(Shift: TShiftState; X, Y: Integer);
var Item: TListItem;
begin
  inherited;
  if not HotTrack then exit;
  Item := GetItemAt(X, Y);
  if Selected <> Item then Selected := Item;
end;

procedure TsohoListView.WMSIZE(var Message: TWMSize);
begin
  inherited;
  RecalcWidth;
end;

procedure TsohoListView.DoBeforeColResize(Column: TListColumn);
begin
  if Assigned(FBeforeColResize) then FBeforeColResize(Self, Column);
end;

procedure TsohoListView.DoAfterColResize(Column: TListColumn);
begin
  if Assigned(FAfterColResize) then FAfterColResize(Self, Column);
end;

procedure TsohoListView.WMNotify(var Message: TWMNotify);
const
  RECURSE_FLAG: boolean = FALSE;var HeaderNotify: PHDNotify;
begin
  if (Message.NMHdr.code < HDN_FIRST) and (Message.NMHdr.code > HDN_LAST) then
    if Message.NMHdr.hwndFrom <> FHeaderHandle then
      FHeaderHandle := Message.NMHdr.hwndFrom;

  // Note the recursion flag.  This is needed since the SetColumnsOwnerDrawFlag
  // call below will cause some HDN_xxx notification messages.
  if RECURSE_FLAG then exit;

  if (not FCanColResize) and
     ((Message.NMHdr^.Code = HDN_BEGINTRACKA) or
      (Message.NMHdr^.Code = HDN_BEGINTRACKW)) then begin
    Message.Result := 1;
    exit;
  end;
  inherited;
  HeaderNotify := PHDNotify(Pointer(Message.NMHdr));
  with Message.NMHdr^ do begin
    if (code = HDN_BEGINTRACK) then
      with HeaderNotify^,  PItem^ do DoBeforeColResize(Column[Item]);
    if (code = HDN_ENDTRACK) then
     with HeaderNotify^,  PItem^ do begin
       DoAfterColResize(Column[Item]);
       if AutoWidth then RecalcWidth;
     end;
  end;
end;

procedure TsohoListView.WMDrawHeader(var Message: TWMDrawItem);
var
  State: TOwnerDrawState;
  DoDefaultDrawing: boolean;
  SavedDC: integer;
begin
  if not Assigned(FCanvas) then exit;

  with Message.DrawItemStruct^ do begin
    Message.Result := 1;
    State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
    SavedDC := SaveDC(hDC);
    FCanvas.Handle := hDC;
    try
      FCanvas.Font := Font;
      FCanvas.Brush := Brush;
      DoDefaultDrawing := FALSE;
      ProcessDrawHeaderMsg(itemID, rcItem, State, DoDefaultDrawing);
    finally
      FCanvas.Handle := 0;
      RestoreDC(hDC, SavedDC);
    end;
  end;
end;

procedure TsohoListView.ProcessDrawHeaderMsg(Index: Integer; Rect: TRect;
   State: TOwnerDrawState; var DefaultDrawing: boolean);
begin
  FCanvas.Font.Assign(Font);
  FCanvas.Brush.Assign(Brush);
  FCanvas.Brush.Style := bsClear;
  FCanvas.Brush.Color := clBtnFace;

  DefaultDrawing := csDesigning in ComponentState;
  if not (csDesigning in ComponentState) then
    DrawHeader(FCanvas, Index, Rect, odSelected in State, DefaultDrawing);

  if DefaultDrawing then
    DefaultDrawHeader(FCanvas, Index, Rect, odSelected in State);
end;

procedure TsohoListView.DefaultDrawHeader(var Canvas: TCanvas;
   Index: Integer; var Rect: TRect; Selected: boolean);
var
  TheColumn: TListColumn;
  Offset: integer;
  R, CR: TRect;
  Bmp: TBitmap;
begin
  if not Selected then  InflateRect(Rect, -2, -2);
  Canvas.FillRect(Rect);
  if Selected then InflateRect(Rect, -2, -2);

  if (Index >= 0) and (Index < Columns.Count) then begin
    // Don't use ActualColumn[] here!  That's for SubItem foolery, not header.
    TheColumn := Columns[Index];

    if Selected then begin
      inc(Rect.Top);
      inc(Rect.Left);
    end;

    R := Rect;

    case TheColumn.Alignment of
      taRightJustify: Dec(R.Right, 4);
      taLeftJustify:  Inc(R.Left, 4);
      // taCenter needs no modification
    end;

    if (FLastSortIndex = Index) then
    begin
      if FAscOrder then Bmp := FUpBmp
      else Bmp := FDownBmp;

      if TheColumn.Alignment = taRightJustify then
        Inc(R.Left, Bmp.Width + 8)
      else
        Dec(R.Right, Bmp.Width + 8);

      { How big of a rectangle do we have to work with for the text? }
      CR := R;
      DrawTextEx(FCanvas.Handle, PChar(TheColumn.Caption), -1, CR,
         DRAWTEXTEX_FLAGS or DT_CALCRECT or
         DRAWTEXTEX_ALIGNMENT[TheColumn.Alignment], NIL);
      { Note that DT_CALCRECT does not adjust for alignment. We must do that }
      case TheColumn.Alignment of
        taRightJustify:
          R.Left := R.Right - (CR.Right - CR.Left);
        taCenter:
          begin
            R.Left := R.Left + (((R.Right - R.Left) - (CR.Right - CR.Left)) div
               2);
            R.Right := R.Left + (CR.Right - CR.Left);
          end;
      else // taLeftJustify: doesn't matter, that is what DT_CALCRECT returns
        R := CR;
      end;
      if R.Left < Rect.Left then
        R.Left := Rect.Left;
      if R.Right > Rect.Right then
        R.Right := Rect.Right;

      if Selected then
        OffsetRect(R, 1, 1);
      // Draw the caption in the rect available
      DrawTextEx(FCanvas.Handle, PChar(TheColumn.Caption), -1, R,
         DRAWTEXTEX_FLAGS or DRAWTEXTEX_ALIGNMENT[TheColumn.Alignment], NIL);

      // Draw the sort arrow bitmap
      Offset := (Rect.Bottom - Rect.Top - Bmp.Height) div 2;
      case TheColumn.Alignment of
        taRightJustify:
          // Only draw if we have enough room
          if (R.Left - Bmp.Width - 8) >= Rect.Left then
            Canvas.Draw(R.Left - Bmp.Width - 8, R.Top + Offset, Bmp);
      else // taLeftJustify, taCenter
        // Only draw if we have enough room
        if (R.Right + Bmp.Width + 8) <= Rect.Right then
          Canvas.Draw(R.Right + 8, R.Top + Offset, Bmp);
      end;
    end else begin
      if Selected then
        OffsetRect(R, 1, 1);
      DrawTextEx(FCanvas.Handle, PChar(TheColumn.Caption), -1, R,
         DRAWTEXTEX_FLAGS or DRAWTEXTEX_ALIGNMENT[TheColumn.Alignment], NIL);
    end;
  end;
end;

procedure TsohoListView.SetColumnsOwnerDrawFlag(OwnerDrawn: boolean);
var
  Item: THDItem;
  x: integer;
begin
  if not HandleAllocated then exit;

  for x := 0 to pred(Columns.Count) do begin
    Item.Mask := HDI_FORMAT;
    if Header_GetItem(HeaderHandle, x, Item) then begin
      if OwnerDrawn then Item.Fmt := Item.Fmt or HDF_OWNERDRAW
      else Item.Fmt := Item.Fmt and not HDF_OWNERDRAW;
      Header_SetItem(HeaderHandle, x, Item);
    end;
  end;

  if not Assigned(FCanvas) then FCanvas := TCanvas.Create;
end;

function TsohoListView.GetHotCursor: TCursor;
var Index : longint;
    Tmp: HCursor;
begin
  Tmp := ListView_GetHotCursor(Handle);
  Result := crDefault;
  for Index := 0 to pred(THackScreen(Screen).FCursorCount) do
    if Screen.Cursors[Index] = Tmp then begin
      Result := Index;
      exit;
    end;
end;

procedure TsohoListView.SetHotCursor (Value: TCursor);
begin
  ListView_SetHotCursor(Handle, Screen.Cursors[Value]);
end;

procedure TsohoListView.SetHotItem (Index: integer);
begin
  ListView_SetHotItem(Handle, Index);
end;

function TsohoListView.GetHotItem : integer;
begin
  Result := ListView_GetHotItem(Handle);
end;

procedure TsohoListView.SetStriply (Value: boolean);
begin
  if FStriply = Value then exit;
  FStriply := Value;
  Invalidate;
end;

procedure TsohoListView.SetStriplyColor (Value: TColor);
begin
  if FStriplyColor = Value then exit;
  FStriplyColor := Value;
  if FStriply then Invalidate;
end;

procedure TsohoListView.SetAutoWidth(Value: boolean);
begin
  if FAutoWidth = Value then exit;
  FAutoWidth := Value;
  if FAutoWidth then RecalcWidth;
end;
{
procedure TsohoListView.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
end;
}

constructor TsohoListView.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FAutoWidth := false;
  FStriply   := false;
  FStriplyColor := $00E8E8E8;
  FCanColResize := true;
  FAutoSort := true;
  FLastSortIndex := -1;
  FUpBmp := TBitmap.Create;
  FUpBmp.OnChange := HeaderBmpChanged;
  FDownBmp := TBitmap.Create;
  FDownBmp.OnChange := HeaderBmpChanged;
  FCanvas := TCanvas.Create;
  CreateSortBmps;
end;

destructor TsohoListView.Destroy;
begin
  FUpBmp.Free;
  FDownBmp.Free;
  if Assigned(FCanvas) then FCanvas.Free;
  inherited Destroy;
end;

procedure TsohoListView.DrawStriply (Item : TListItem; var BkColor : TColor);
begin
  if not FStriply then exit;
  if odd(Item.Index) then BkColor := FStriplyColor;
end;

procedure TsohoListView.CNNotify;

  procedure CallPaintEvent(PaintEvent: TsohoLVPaintEvent);
  var
    Canvas: TCanvas;
  begin
    Canvas := TCanvas.Create;
    with TNMLVCUSTOMDRAW(Pointer(Msg.NMHdr)^), nmcd do
    try
      Canvas.Handle := hdc;
      Canvas.Brush := Self.Brush;
      Canvas.Font := Font;
      PaintEvent(Self,
        TListItem(lItemlParam),
        Canvas,
        rc,
        dwItemSpec,
        uItemState);
    finally
      Canvas.Free;
    end;
  end;

begin
  inherited;
  with Msg, TNMLVCUSTOMDRAW(Pointer(NMHdr)^), nmcd do
  case Hdr.Code of
    NM_CUSTOMDRAW:
    begin
      case dwDrawStage of
        CDDS_PREPAINT: Result := CDRF_NOTIFYITEMDRAW;
        CDDS_ITEMPREPAINT: begin
          DrawStriply(TListItem(lItemlParam), clrTextBk);
          if Assigned(FOnPrePaint) then
            FOnPrePaint(Self,
              TListItem(lItemlParam),
              dwItemSpec,
              uItemState,
              clrText, clrTextBk);
          Result := Result or CDRF_NEWFONT;
          if Assigned(FOnPaint) then begin
            CallPaintEvent(FOnPaint);
            Result := Result or CDRF_SKIPDEFAULT;
          end;
          if Assigned(FOnPostPaint) then
            Result := Result or CDRF_NOTIFYPOSTPAINT;
        end;
        CDDS_ITEMPOSTPAINT:
          if Assigned(FOnPostPaint) then CallPaintEvent(FOnPostPaint);
      end;
    end;
  end;
end;

procedure TsohoListView.HeaderBmpChanged (Sender: TObject);
begin
  Invalidate;
end;

procedure TsohoListView.SetUpBmp (Value: TBitmap);
begin
  FUpBmp.Assign(Value);
end;

procedure TsohoListView.SetDownBmp (Value: TBitmap);
begin
  FDownBmp.Assign(Value);
end;

procedure TsohoListView.CreateSortBmps;
var HeaderHeight: integer;
    MidPoint: integer;
    Bmp: TBitmap;
begin
  FUpBmp.Canvas.Font.Assign(Font);
  HeaderHeight := FUpBmp.Canvas.TextHeight('W') - 6;
  if HeaderHeight > 0 then begin
    if Odd(HeaderHeight) then Inc(HeaderHeight);
    FUpBmp.Width    := HeaderHeight;
    FUpBmp.Height   := HeaderHeight;
    FDownBmp.Width  := HeaderHeight;
    FDownBmp.Height := HeaderHeight;
    MidPoint       := HeaderHeight div 2;

    { Don't ask about the drawing.  I just fooled around until I got
      something I liked. }
    {if FReverseSortArrows then }Bmp := FUpBmp;
    //else Bmp := DownBmp;
    with Bmp.Canvas do begin
      Brush.Color := clBtnFace;
      FillRect(Rect(0, 0, HeaderHeight, HeaderHeight));
      Pen.Color := clBtnHighlight;
      MoveTo(MidPoint, HeaderHeight-2);
      LineTo(HeaderHeight-1, 0);
      Pixels[HeaderHeight-1, 0] := Pen.Color;
      Pen.Color := clBtnShadow;
      MoveTo(HeaderHeight-2, 0);
      LineTo(0, 0);
      LineTo(MidPoint-1, HeaderHeight-2);
      Pixels[MidPoint-1, HeaderHeight-2] := Pen.Color;
    end;

    {if FReverseSortArrows then }Bmp := FDownBmp;
    // else Bmp := UpBmp;
    with Bmp.Canvas do begin
      Brush.Color := clBtnFace;
      FillRect(Rect(0, 0, HeaderHeight, HeaderHeight));
      Pen.Color := clBtnShadow;
      MoveTo(0, HeaderHeight-1);
      LineTo(MidPoint-1, 0);
      Pen.Color := clBtnHighlight;
      MoveTo(MidPoint, 0);
      LineTo(HeaderHeight-1, HeaderHeight-1);
      LineTo(-1, HeaderHeight-1);
      Pixels[MidPoint, 0] := clBtnFace;
    end;
  end;
end;

procedure TsohoListView.InvalidateColumnHeader(Index: integer);
var
  R: TRect;
  //x: integer;
  //w: integer;
begin
  if (Index < 0) or (Index >= Columns.Count) or (HeaderHandle = 0) then
    exit;

  // w := Columns[Index].Width;
  // We have to turn this into the actual column index if drag-drop headers have
  // re-arranged stuff in the TExtListView descendant component.
  // Index := ActualColumnIndex(Index);

  Windows.GetClientRect(HeaderHandle, R);
  (*
  for x := 0 to pred(Columns.Count) do
    {if ActualColumnIndex(x) < Index then
      inc(R.Left, RealColWidth(x));}
  R.Right := R.Left + Columns[x].Width; {EnhListView}
  *)
  // Adjust for shadow
  InflateRect(R, -2, -2);
  InvalidateRect(HeaderHandle, @R, false);
end;

procedure TsohoListView.WMOwnerDrawColumns(var Message: TMessage);
begin
  SetColumnsOwnerDrawFlag(true);
  Update;
end;

procedure TsohoListView.DrawHeader(var Canvas: TCanvas; Index: Integer;
   var Rect: TRect; Selected: boolean; var DefaultDrawing: boolean);
begin
  DefaultDrawing := true;
  {if assigned(FOnDrawHeader) then
    FOnDrawHeader(Self, Canvas, Index, Rect, Selected, DefaultDrawing);}
end;

//------------------------   -------------------
Procedure TsohoListView.SetColumnSortType(Value : TColumnSortType);
Begin
  If Value<>FColumnSortType Then FColumnSortType:=Value;
End; {--- SetColumnSortType ---}

Procedure TsohoListView.SetSortMetod(Value : TSelectSortMetod);
Begin
  If Value<>FSelectSortMetod Then FSelectSortMetod:=Value;
End; {--- SetSortMetod ---}
end.

